@@ -31,8 +31,6 @@
 
 #include "dialogs/dialogs.h" /* FIXME, we are in the widget layer */
 
-#include "menus/menus.h"
-
 #include "config/gimpguiconfig.h"
 
 #include "core/gimp.h"
@@ -128,6 +126,7 @@ static void            gimp_dock_window_style_set                 (GtkWidget
 static gboolean        gimp_dock_window_delete_event              (GtkWidget                  *widget,
                                                                    GdkEventAny                *event);
 static GList         * gimp_dock_window_get_docks                 (GimpDockContainer          *dock_container);
+static GimpDialogFactory * gimp_dock_window_get_dialog_factory    (GimpDockContainer          *dock_container);
 static GimpUIManager * gimp_dock_window_get_ui_manager            (GimpDockContainer          *dock_container);
 static void            gimp_dock_window_add_dock_from_session     (GimpDockContainer          *dock_container,
                                                                    GimpDock                   *dock,
@@ -253,30 +252,29 @@ gimp_dock_window_init (GimpDockWindow *dock_window)
   static gint  dock_window_ID = 1;
   gchar       *name           = NULL;
 
-  /* Initialize members */
   dock_window->p = G_TYPE_INSTANCE_GET_PRIVATE (dock_window,
                                                 GIMP_TYPE_DOCK_WINDOW,
                                                 GimpDockWindowPrivate);
   dock_window->p->ID                 = dock_window_ID++;
   dock_window->p->auto_follow_active = TRUE;
 
-  /* Initialize theming and style-setting stuff */
   name = g_strdup_printf ("gimp-dock-%d", dock_window->p->ID);
   gtk_widget_set_name (GTK_WIDGET (dock_window), name);
   g_free (name);
 
-  /* Misc */
   gtk_window_set_resizable (GTK_WINDOW (dock_window), TRUE);
   gtk_window_set_focus_on_map (GTK_WINDOW (dock_window), FALSE);
+  gtk_window_set_skip_taskbar_hint (GTK_WINDOW (dock_window), FALSE);
 }
 
 static void
 gimp_dock_window_dock_container_iface_init (GimpDockContainerInterface *iface)
 {
-  iface->get_docks      = gimp_dock_window_get_docks;
-  iface->get_ui_manager = gimp_dock_window_get_ui_manager;
-  iface->add_dock       = gimp_dock_window_add_dock_from_session;
-  iface->get_dock_side  = gimp_dock_window_get_dock_side;
+  iface->get_docks          = gimp_dock_window_get_docks;
+  iface->get_dialog_factory = gimp_dock_window_get_dialog_factory;
+  iface->get_ui_manager     = gimp_dock_window_get_ui_manager;
+  iface->add_dock           = gimp_dock_window_add_dock_from_session;
+  iface->get_dock_side      = gimp_dock_window_get_dock_side;
 }
 
 static void
@@ -289,16 +287,17 @@ gimp_dock_window_session_managed_iface_init (GimpSessionManagedInterface *iface)
 static void
 gimp_dock_window_constructed (GObject *object)
 {
-  GimpDockWindow *dock_window = GIMP_DOCK_WINDOW (object);
-  GimpGuiConfig  *config;
-  GtkAccelGroup  *accel_group;
-  Gimp           *gimp;
-  GtkSettings    *settings;
-  gint            menu_view_width  = -1;
-  gint            menu_view_height = -1;
+  GimpDockWindow  *dock_window = GIMP_DOCK_WINDOW (object);
+  GimpGuiConfig   *config;
+  GimpContext     *factory_context;
+  GimpMenuFactory *menu_factory;
+  GtkAccelGroup   *accel_group;
+  Gimp            *gimp;
+  GtkSettings     *settings;
+  gint             menu_view_width  = -1;
+  gint             menu_view_height = -1;
 
-  if (G_OBJECT_CLASS (parent_class)->constructed)
-    G_OBJECT_CLASS (parent_class)->constructed (object);
+  G_OBJECT_CLASS (parent_class)->constructed (object);
 
   gimp   = GIMP (dock_window->p->context->gimp);
   config = GIMP_GUI_CONFIG (gimp->config);
@@ -312,18 +311,24 @@ gimp_dock_window_constructed (GObject *object)
   dock_window->p->image_container   = gimp->images;
   dock_window->p->display_container = gimp->displays;
 
+  factory_context =
+    gimp_dialog_factory_get_context (dock_window->p->dialog_factory);
+
   /* Setup hints */
   gimp_window_set_hint (GTK_WINDOW (dock_window), config->dock_window_hint);
 
+  menu_factory =
+    gimp_dialog_factory_get_menu_factory (dock_window->p->dialog_factory);
+
   /* Make image window related keyboard shortcuts work also when a
    * dock window is the focused window
    */
   dock_window->p->ui_manager =
-    gimp_menu_factory_manager_new (global_menu_factory,
+    gimp_menu_factory_manager_new (menu_factory,
                                    dock_window->p->ui_manager_name,
                                    dock_window,
                                    config->tearoff_menus);
-  accel_group = 
+  accel_group =
     gtk_ui_manager_get_accel_group (GTK_UI_MANAGER (dock_window->p->ui_manager));
   gtk_window_add_accel_group (GTK_WINDOW (dock_window), accel_group);
 
@@ -342,12 +347,12 @@ gimp_dock_window_constructed (GObject *object)
                                 dock_window);
 
   gimp_context_define_properties (dock_window->p->context,
-                                  GIMP_CONTEXT_ALL_PROPS_MASK &
-                                  ~(GIMP_CONTEXT_IMAGE_MASK |
-                                    GIMP_CONTEXT_DISPLAY_MASK),
+                                  GIMP_CONTEXT_PROP_MASK_ALL &
+                                  ~(GIMP_CONTEXT_PROP_MASK_IMAGE |
+                                    GIMP_CONTEXT_PROP_MASK_DISPLAY),
                                   FALSE);
   gimp_context_set_parent (dock_window->p->context,
-                           gimp_dialog_factory_get_context (dock_window->p->dialog_factory));
+                           factory_context);
 
   /* Setup widget hierarchy */
   {
@@ -390,7 +395,7 @@ gimp_dock_window_constructed (GObject *object)
                         dock_window);
 
       gimp_help_set_help_data (dock_window->p->auto_button,
-                               _("When enabled the dialog automatically "
+                               _("When enabled, the dialog automatically "
                                  "follows the image you are working on."),
                                GIMP_HELP_DOCK_AUTO_BUTTON);
     }
@@ -424,21 +429,21 @@ gimp_dock_window_constructed (GObject *object)
 
   if (dock_window->p->auto_follow_active)
     {
-      if (gimp_context_get_display (gimp_dialog_factory_get_context (dock_window->p->dialog_factory)))
-        gimp_context_copy_property (gimp_dialog_factory_get_context (dock_window->p->dialog_factory),
+      if (gimp_context_get_display (factory_context))
+        gimp_context_copy_property (factory_context,
                                     dock_window->p->context,
                                     GIMP_CONTEXT_PROP_DISPLAY);
       else
-        gimp_context_copy_property (gimp_dialog_factory_get_context (dock_window->p->dialog_factory),
+        gimp_context_copy_property (factory_context,
                                     dock_window->p->context,
                                     GIMP_CONTEXT_PROP_IMAGE);
     }
 
-  g_signal_connect_object (gimp_dialog_factory_get_context (dock_window->p->dialog_factory), "display-changed",
+  g_signal_connect_object (factory_context, "display-changed",
                            G_CALLBACK (gimp_dock_window_factory_display_changed),
                            dock_window,
                            0);
-  g_signal_connect_object (gimp_dialog_factory_get_context (dock_window->p->dialog_factory), "image-changed",
+  g_signal_connect_object (factory_context, "image-changed",
                            G_CALLBACK (gimp_dock_window_factory_image_changed),
                            dock_window,
                            0);
@@ -459,12 +464,12 @@ gimp_dock_window_constructed (GObject *object)
 
   if (dock_window->p->auto_follow_active)
     {
-      if (gimp_context_get_display (gimp_dialog_factory_get_context (dock_window->p->dialog_factory)))
-        gimp_context_copy_property (gimp_dialog_factory_get_context (dock_window->p->dialog_factory),
+      if (gimp_context_get_display (factory_context))
+        gimp_context_copy_property (factory_context,
                                     dock_window->p->context,
                                     GIMP_CONTEXT_PROP_DISPLAY);
       else
-        gimp_context_copy_property (gimp_dialog_factory_get_context (dock_window->p->dialog_factory),
+        gimp_context_copy_property (factory_context,
                                     dock_window->p->context,
                                     GIMP_CONTEXT_PROP_IMAGE);
     }
@@ -690,7 +695,8 @@ gimp_dock_window_delete_event (GtkWidget   *widget,
   entry_name = (gimp_dock_window_has_toolbox (dock_window) ?
                 "gimp-toolbox-window" :
                 "gimp-dock-window");
-  entry = gimp_dialog_factory_find_entry (gimp_dialog_factory_get_singleton (), entry_name);
+  entry = gimp_dialog_factory_find_entry (dock_window->p->dialog_factory,
+                                          entry_name);
   gimp_session_info_set_factory_entry (info, entry);
 
   gimp_container_add (global_recent_docks, GIMP_OBJECT (info));
@@ -702,23 +708,23 @@ gimp_dock_window_delete_event (GtkWidget   *widget,
 static GList *
 gimp_dock_window_get_docks (GimpDockContainer *dock_container)
 {
-  GimpDockWindow *dock_window;
-
-  g_return_val_if_fail (GIMP_IS_DOCK_WINDOW (dock_container), NULL);
-
-  dock_window = GIMP_DOCK_WINDOW (dock_container);
+  GimpDockWindow *dock_window = GIMP_DOCK_WINDOW (dock_container);
 
   return g_list_copy (gimp_dock_columns_get_docks (dock_window->p->dock_columns));
 }
 
+static GimpDialogFactory *
+gimp_dock_window_get_dialog_factory (GimpDockContainer *dock_container)
+{
+  GimpDockWindow *dock_window = GIMP_DOCK_WINDOW (dock_container);
+
+  return dock_window->p->dialog_factory;
+}
+
 static GimpUIManager *
 gimp_dock_window_get_ui_manager (GimpDockContainer *dock_container)
 {
-  GimpDockWindow *dock_window;
-
-  g_return_val_if_fail (GIMP_IS_DOCK_WINDOW (dock_container), NULL);
-
-  dock_window = GIMP_DOCK_WINDOW (dock_container);
+  GimpDockWindow *dock_window = GIMP_DOCK_WINDOW (dock_container);
 
   return dock_window->p->ui_manager;
 }
@@ -728,11 +734,7 @@ gimp_dock_window_add_dock_from_session (GimpDockContainer   *dock_container,
                                         GimpDock            *dock,
                                         GimpSessionInfoDock *dock_info)
 {
-  GimpDockWindow *dock_window;
-
-  g_return_if_fail (GIMP_IS_DOCK_WINDOW (dock_container));
-
-  dock_window = GIMP_DOCK_WINDOW (dock_container);
+  GimpDockWindow *dock_window = GIMP_DOCK_WINDOW (dock_container);
 
   gimp_dock_window_add_dock (dock_window,
                              dock,
@@ -742,14 +744,10 @@ gimp_dock_window_add_dock_from_session (GimpDockContainer   *dock_container,
 static GList *
 gimp_dock_window_get_aux_info (GimpSessionManaged *session_managed)
 {
-  GimpDockWindow     *dock_window;
-  GList              *aux_info = NULL;
+  GimpDockWindow     *dock_window = GIMP_DOCK_WINDOW (session_managed);
+  GList              *aux_info    = NULL;
   GimpSessionInfoAux *aux;
 
-  g_return_val_if_fail (GIMP_IS_DOCK_WINDOW (session_managed), NULL);
-
-  dock_window = GIMP_DOCK_WINDOW (session_managed);
-
   if (dock_window->p->allow_dockbook_absence)
     {
       /* Assume it is the toolbox; it does not have aux info */
@@ -862,15 +860,6 @@ gimp_dock_window_should_add_to_recent (GimpDockWindow *dock_window)
   return should_add;
 }
 
-static void
-gimp_dock_window_display_changed (GimpDockWindow *dock_window,
-                                  GimpObject     *display,
-                                  GimpContext    *context)
-{
-  gimp_ui_manager_update (dock_window->p->ui_manager,
-                          display);
-}
-
 static void
 gimp_dock_window_image_flush (GimpImage      *image,
                               gboolean        invalidate_preview,
@@ -974,6 +963,24 @@ gimp_dock_window_factory_image_changed (GimpContext *context,
     gimp_context_set_image (dock_window->p->context, image);
 }
 
+static void
+gimp_dock_window_display_changed (GimpDockWindow *dock_window,
+                                  GimpObject     *display,
+                                  GimpContext    *context)
+{
+  /*  make sure auto-follow-active works both ways  */
+  if (display && dock_window->p->auto_follow_active)
+    {
+      GimpContext *factory_context =
+        gimp_dialog_factory_get_context (dock_window->p->dialog_factory);
+
+      gimp_context_set_display (factory_context, display);
+    }
+
+  gimp_ui_manager_update (dock_window->p->ui_manager,
+                          display);
+}
+
 static void
 gimp_dock_window_image_changed (GimpDockWindow *dock_window,
                                 GimpImage      *image,
@@ -982,6 +989,15 @@ gimp_dock_window_image_changed (GimpDockWindow *dock_window,
   GimpContainer *image_container   = dock_window->p->image_container;
   GimpContainer *display_container = dock_window->p->display_container;
 
+  /*  make sure auto-follow-active works both ways  */
+  if (image && dock_window->p->auto_follow_active)
+    {
+      GimpContext *factory_context =
+        gimp_dialog_factory_get_context (dock_window->p->dialog_factory);
+
+      gimp_context_set_image (factory_context, image);
+    }
+
   if (image == NULL && ! gimp_container_is_empty (image_container))
     {
       image = GIMP_IMAGE (gimp_container_get_first_child (image_container));
@@ -1021,7 +1037,7 @@ gimp_dock_window_image_changed (GimpDockWindow *dock_window,
         {
           GList *list;
 
-          for (list = GIMP_LIST (display_container)->list;
+          for (list = GIMP_LIST (display_container)->queue->head;
                list;
                list = g_list_next (list))
             {
@@ -1066,10 +1082,14 @@ gimp_dock_window_auto_clicked (GtkWidget *widget,
 
   if (dock_window->p->auto_follow_active)
     {
-      gimp_context_copy_properties (gimp_dialog_factory_get_context (dock_window->p->dialog_factory),
+      GimpContext *context;
+
+      context = gimp_dialog_factory_get_context (dock_window->p->dialog_factory);
+
+      gimp_context_copy_properties (context,
                                     dock_window->p->context,
-                                    GIMP_CONTEXT_DISPLAY_MASK |
-                                    GIMP_CONTEXT_IMAGE_MASK);
+                                    GIMP_CONTEXT_PROP_MASK_DISPLAY |
+                                    GIMP_CONTEXT_PROP_MASK_IMAGE);
     }
 }
 
@@ -1119,17 +1139,17 @@ GtkWidget *
 gimp_dock_window_new (const gchar       *role,
                       const gchar       *ui_manager_name,
                       gboolean           allow_dockbook_absence,
-                      GimpDialogFactory *factory,
+                      GimpDialogFactory *dialog_factory,
                       GimpContext       *context)
 {
-  g_return_val_if_fail (GIMP_IS_DIALOG_FACTORY (factory), NULL);
+  g_return_val_if_fail (GIMP_IS_DIALOG_FACTORY (dialog_factory), NULL);
   g_return_val_if_fail (GIMP_IS_CONTEXT (context), NULL);
 
   return g_object_new (GIMP_TYPE_DOCK_WINDOW,
                        "role",                   role,
                        "ui-manager-name",        ui_manager_name,
                        "allow-dockbook-absence", allow_dockbook_absence,
-                       "dialog-factory",         factory,
+                       "dialog-factory",         dialog_factory,
                        "context",                context,
                        NULL);
 }
@@ -1150,14 +1170,6 @@ gimp_dock_window_get_context (GimpDockWindow *dock_window)
   return dock_window->p->context;
 }
 
-GimpDialogFactory *
-gimp_dock_window_get_dialog_factory (GimpDockWindow *dock_window)
-{
-  g_return_val_if_fail (GIMP_IS_DOCK_WINDOW (dock_window), NULL);
-
-  return dock_window->p->dialog_factory;
-}
-
 gboolean
 gimp_dock_window_get_auto_follow_active (GimpDockWindow *dock_window)
 {
