@@ -1,9 +1,9 @@
 /* GIMP - The GNU Image Manipulation Program
  * Copyright (C) 1995 Spencer Kimball and Peter Mattis
  *
- * This program is free software; you can redistribute it and/or modify
+ * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -12,23 +12,22 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
 
+#include <gegl.h>
 #include <gtk/gtk.h>
 
 #include "libgimpwidgets/gimpwidgets.h"
 
 #include "widgets-types.h"
 
-#include "base/gimphistogram.h"
-
 #include "core/gimp.h"
 #include "core/gimpdrawable.h"
 #include "core/gimpdrawable-histogram.h"
+#include "core/gimphistogram.h"
 #include "core/gimpimage.h"
 
 #include "gimpdocked.h"
@@ -42,8 +41,24 @@
 #include "gimp-intl.h"
 
 
+enum
+{
+  PROP_0,
+  PROP_LINEAR
+};
+
+
 static void     gimp_histogram_editor_docked_iface_init (GimpDockedInterface *iface);
 
+static void     gimp_histogram_editor_set_property  (GObject            *object,
+                                                     guint               property_id,
+                                                     const GValue       *value,
+                                                     GParamSpec         *pspec);
+static void     gimp_histogram_editor_get_property  (GObject            *object,
+                                                     guint               property_id,
+                                                     GValue             *value,
+                                                     GParamSpec         *pspec);
+
 static void     gimp_histogram_editor_set_aux_info  (GimpDocked          *docked,
                                                      GList               *aux_info);
 static GList  * gimp_histogram_editor_get_aux_info  (GimpDocked          *docked);
@@ -54,6 +69,8 @@ static void     gimp_histogram_editor_layer_changed (GimpImage           *image,
                                                      GimpHistogramEditor *editor);
 static void     gimp_histogram_editor_frozen_update (GimpHistogramEditor *editor,
                                                      const GParamSpec    *pspec);
+static void     gimp_histogram_editor_buffer_update (GimpHistogramEditor *editor,
+                                                     const GParamSpec    *pspec);
 static void     gimp_histogram_editor_update        (GimpHistogramEditor *editor);
 
 static gboolean gimp_histogram_editor_idle_update   (GimpHistogramEditor *editor);
@@ -79,9 +96,20 @@ static GimpDockedInterface *parent_docked_iface = NULL;
 static void
 gimp_histogram_editor_class_init (GimpHistogramEditorClass *klass)
 {
+  GObjectClass         *object_class       = G_OBJECT_CLASS (klass);
   GimpImageEditorClass *image_editor_class = GIMP_IMAGE_EDITOR_CLASS (klass);
 
+  object_class->set_property    = gimp_histogram_editor_set_property;
+  object_class->get_property    = gimp_histogram_editor_get_property;
+
   image_editor_class->set_image = gimp_histogram_editor_set_image;
+
+  g_object_class_install_property (object_class, PROP_LINEAR,
+                                   g_param_spec_boolean ("linear",
+                                                         _("Linear"), NULL,
+                                                         TRUE,
+                                                         GIMP_PARAM_READWRITE |
+                                                         G_PARAM_CONSTRUCT));
 }
 
 static void
@@ -91,6 +119,7 @@ gimp_histogram_editor_init (GimpHistogramEditor *editor)
   GtkWidget         *hbox;
   GtkWidget         *label;
   GtkWidget         *menu;
+  GtkWidget         *button;
   GtkWidget         *table;
   gint               i;
 
@@ -115,19 +144,15 @@ gimp_histogram_editor_init (GimpHistogramEditor *editor)
 
   view = GIMP_HISTOGRAM_BOX (editor->box)->view;
 
-  hbox = gtk_hbox_new (FALSE, 6);
+  hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 6);
   gtk_box_pack_start (GTK_BOX (editor), hbox, FALSE, FALSE, 0);
   gtk_widget_show (hbox);
 
-  label = gtk_label_new (_("Channel:"));
-  gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
-  gtk_widget_show (label);
-
   editor->menu = menu = gimp_prop_enum_combo_box_new (G_OBJECT (view),
                                                       "histogram-channel",
                                                       0, 0);
-  gimp_enum_combo_box_set_stock_prefix (GIMP_ENUM_COMBO_BOX (menu),
-                                        "gimp-channel");
+  gimp_enum_combo_box_set_icon_prefix (GIMP_ENUM_COMBO_BOX (menu),
+                                       "gimp-channel");
   gimp_int_combo_box_set_sensitivity (GIMP_INT_COMBO_BOX (editor->menu),
                                       gimp_histogram_menu_sensitivity,
                                       editor, NULL);
@@ -136,12 +161,26 @@ gimp_histogram_editor_init (GimpHistogramEditor *editor)
   gtk_box_pack_start (GTK_BOX (hbox), menu, FALSE, FALSE, 0);
   gtk_widget_show (menu);
 
-  menu = gimp_prop_enum_stock_box_new (G_OBJECT (view),
-                                       "histogram-scale", "gimp-histogram",
-                                       0, 0);
+  gimp_help_set_help_data (editor->menu,
+                           _("Histogram channel"), NULL);
+
+  menu = gimp_prop_enum_icon_box_new (G_OBJECT (view),
+                                      "histogram-scale", "gimp-histogram",
+                                      0, 0);
   gtk_box_pack_end (GTK_BOX (hbox), menu, FALSE, FALSE, 0);
   gtk_widget_show (menu);
 
+  button = gimp_prop_check_button_new (G_OBJECT (editor), "linear", NULL);
+  gtk_toggle_button_set_mode (GTK_TOGGLE_BUTTON (button), FALSE);
+  gtk_box_pack_end (GTK_BOX (hbox), button, FALSE, FALSE, 0);
+  gtk_widget_show (button);
+
+  gimp_help_set_help_data (button,
+                           "This button switches between a histogram in "
+                           "sRGB (gamma corrected) space and a histogram "
+                           "in linear space. Also, it's horrible UI and "
+                           "needs to be changed.", NULL);
+
   gtk_box_pack_start (GTK_BOX (editor), editor->box, TRUE, TRUE, 0);
   gtk_widget_show (GTK_WIDGET (editor->box));
 
@@ -172,7 +211,7 @@ gimp_histogram_editor_init (GimpHistogramEditor *editor)
                                  PANGO_ATTR_WEIGHT, PANGO_WEIGHT_BOLD,
                                  PANGO_ATTR_SCALE,  PANGO_SCALE_SMALL,
                                  -1);
-      gtk_misc_set_alignment (GTK_MISC (label), 1.0, 0.5);
+      gtk_label_set_xalign (GTK_LABEL (label), 1.0);
       gtk_table_attach (GTK_TABLE (table), label, x, x + 1, y, y + 1,
                         GTK_FILL | GTK_EXPAND, GTK_FILL, 2, 2);
       gtk_widget_show (label);
@@ -204,6 +243,65 @@ gimp_histogram_editor_docked_iface_init (GimpDockedInterface *docked_iface)
   docked_iface->get_aux_info = gimp_histogram_editor_get_aux_info;
 }
 
+static void
+gimp_histogram_editor_set_property (GObject      *object,
+                                    guint         property_id,
+                                    const GValue *value,
+                                    GParamSpec   *pspec)
+{
+  GimpHistogramEditor *editor = GIMP_HISTOGRAM_EDITOR (object);
+  GimpHistogramView   *view   = GIMP_HISTOGRAM_BOX (editor->box)->view;
+
+  switch (property_id)
+    {
+    case PROP_LINEAR:
+      editor->linear = g_value_get_boolean (value);
+
+      if (editor->histogram)
+        {
+          g_object_unref (editor->histogram);
+          editor->histogram = NULL;
+
+          gimp_histogram_view_set_histogram (view, NULL);
+        }
+
+      if (editor->bg_histogram)
+        {
+          g_object_unref (editor->bg_histogram);
+          editor->bg_histogram = NULL;
+
+          gimp_histogram_view_set_background (view, NULL);
+        }
+
+      gimp_histogram_editor_update (editor);
+      break;
+
+   default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
+      break;
+    }
+}
+
+static void
+gimp_histogram_editor_get_property (GObject    *object,
+                                    guint       property_id,
+                                    GValue     *value,
+                                    GParamSpec *pspec)
+{
+  GimpHistogramEditor *editor = GIMP_HISTOGRAM_EDITOR (object);
+
+  switch (property_id)
+    {
+    case PROP_LINEAR:
+      g_value_set_boolean (value, editor->linear);
+      break;
+
+   default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
+      break;
+    }
+}
+
 static void
 gimp_histogram_editor_set_aux_info (GimpDocked *docked,
                                     GList      *aux_info)
@@ -250,6 +348,9 @@ gimp_histogram_editor_set_image (GimpImageEditor *image_editor,
           editor->idle_id = 0;
         }
 
+      g_signal_handlers_disconnect_by_func (image_editor->image,
+                                            gimp_histogram_editor_update,
+                                            editor);
       g_signal_handlers_disconnect_by_func (image_editor->image,
                                             gimp_histogram_editor_layer_changed,
                                             editor);
@@ -259,7 +360,7 @@ gimp_histogram_editor_set_image (GimpImageEditor *image_editor,
 
       if (editor->histogram)
         {
-          gimp_histogram_unref (editor->histogram);
+          g_object_unref (editor->histogram);
           editor->histogram = NULL;
 
           gimp_histogram_view_set_histogram (view, NULL);
@@ -267,7 +368,7 @@ gimp_histogram_editor_set_image (GimpImageEditor *image_editor,
 
       if (editor->bg_histogram)
         {
-          gimp_histogram_unref (editor->bg_histogram);
+          g_object_unref (editor->bg_histogram);
           editor->bg_histogram = NULL;
 
           gimp_histogram_view_set_background (view, NULL);
@@ -278,10 +379,6 @@ gimp_histogram_editor_set_image (GimpImageEditor *image_editor,
 
   if (image)
     {
-      editor->histogram = gimp_histogram_new ();
-
-      gimp_histogram_view_set_histogram (view, editor->histogram);
-
       g_signal_connect_object (image, "mode-changed",
                                G_CALLBACK (gimp_histogram_editor_menu_update),
                                editor, G_CONNECT_SWAPPED);
@@ -308,11 +405,19 @@ gimp_histogram_editor_layer_changed (GimpImage           *image,
 {
   if (editor->drawable)
     {
+      GimpHistogramView *view = GIMP_HISTOGRAM_BOX (editor->box)->view;
+
+      if (editor->histogram)
+        {
+          g_object_unref (editor->histogram);
+          editor->histogram = NULL;
+
+          gimp_histogram_view_set_histogram (view, NULL);
+        }
+
       if (editor->bg_histogram)
         {
-          GimpHistogramView *view = GIMP_HISTOGRAM_BOX (editor->box)->view;
-
-          gimp_histogram_unref (editor->bg_histogram);
+          g_object_unref (editor->bg_histogram);
           editor->bg_histogram = NULL;
 
           gimp_histogram_view_set_background (view, NULL);
@@ -327,6 +432,9 @@ gimp_histogram_editor_layer_changed (GimpImage           *image,
       g_signal_handlers_disconnect_by_func (editor->drawable,
                                             gimp_histogram_editor_update,
                                             editor);
+      g_signal_handlers_disconnect_by_func (editor->drawable,
+                                            gimp_histogram_editor_buffer_update,
+                                            editor);
       g_signal_handlers_disconnect_by_func (editor->drawable,
                                             gimp_histogram_editor_frozen_update,
                                             editor);
@@ -343,6 +451,9 @@ gimp_histogram_editor_layer_changed (GimpImage           *image,
       g_signal_connect_object (editor->drawable, "notify::frozen",
                                G_CALLBACK (gimp_histogram_editor_frozen_update),
                                editor, G_CONNECT_SWAPPED);
+      g_signal_connect_object (editor->drawable, "notify::buffer",
+                               G_CALLBACK (gimp_histogram_editor_buffer_update),
+                               editor, G_CONNECT_SWAPPED);
       g_signal_connect_object (editor->drawable, "update",
                                G_CALLBACK (gimp_histogram_editor_update),
                                editor, G_CONNECT_SWAPPED);
@@ -353,7 +464,7 @@ gimp_histogram_editor_layer_changed (GimpImage           *image,
                                G_CALLBACK (gimp_histogram_editor_name_update),
                                editor, G_CONNECT_SWAPPED);
 
-      gimp_histogram_editor_update (editor);
+      gimp_histogram_editor_buffer_update (editor, NULL);
     }
   else if (editor->histogram)
     {
@@ -368,16 +479,32 @@ gimp_histogram_editor_layer_changed (GimpImage           *image,
 static gboolean
 gimp_histogram_editor_validate (GimpHistogramEditor *editor)
 {
-  if (! editor->valid && editor->histogram)
+  if (! editor->valid)
     {
       if (editor->drawable)
-        gimp_drawable_calculate_histogram (editor->drawable, editor->histogram);
+        {
+          if (! editor->histogram)
+            {
+              GimpHistogramView *view = GIMP_HISTOGRAM_BOX (editor->box)->view;
+
+              editor->histogram = gimp_histogram_new (editor->linear);
+
+              gimp_histogram_view_set_histogram (view, editor->histogram);
+            }
+
+          gimp_drawable_calculate_histogram (editor->drawable,
+                                             editor->histogram);
+        }
       else
-        gimp_histogram_calculate (editor->histogram, NULL, NULL);
+        {
+          if (editor->histogram)
+            gimp_histogram_clear_values (editor->histogram);
+        }
 
       gimp_histogram_editor_info_update (editor);
 
-      editor->valid = TRUE;
+      if (editor->histogram)
+        editor->valid = TRUE;
     }
 
   return editor->valid;
@@ -397,7 +524,8 @@ gimp_histogram_editor_frozen_update (GimpHistogramEditor *editor,
        * is shown. So don't slow down painting by doing something that
        * is not even seen by the user.
        */
-      if (! editor->bg_histogram && GTK_WIDGET_DRAWABLE (editor))
+      if (! editor->bg_histogram &&
+          gtk_widget_is_drawable (GTK_WIDGET (editor)))
         {
           if (gimp_histogram_editor_validate (editor))
             editor->bg_histogram = gimp_histogram_duplicate (editor->histogram);
@@ -407,13 +535,22 @@ gimp_histogram_editor_frozen_update (GimpHistogramEditor *editor,
     }
   else if (editor->bg_histogram)
     {
-      gimp_histogram_unref (editor->bg_histogram);
+      g_object_unref (editor->bg_histogram);
       editor->bg_histogram = NULL;
 
       gimp_histogram_view_set_background (view, NULL);
     }
 }
 
+static void
+gimp_histogram_editor_buffer_update (GimpHistogramEditor *editor,
+                                     const GParamSpec    *pspec)
+{
+  g_object_set (editor,
+                "linear", gimp_drawable_get_linear (editor->drawable),
+                NULL);
+}
+
 static void
 gimp_histogram_editor_update (GimpHistogramEditor *editor)
 {
@@ -457,6 +594,7 @@ gimp_histogram_editor_channel_valid (GimpHistogramEditor  *editor,
         case GIMP_HISTOGRAM_RED:
         case GIMP_HISTOGRAM_GREEN:
         case GIMP_HISTOGRAM_BLUE:
+        case GIMP_HISTOGRAM_LUMINANCE:
         case GIMP_HISTOGRAM_RGB:
           return gimp_drawable_is_rgb (editor->drawable);
 
@@ -500,7 +638,7 @@ gimp_histogram_editor_name_update (GimpHistogramEditor *editor)
   const gchar *name = NULL;
 
   if (editor->drawable)
-    name = gimp_object_get_name (GIMP_OBJECT (editor->drawable));
+    name = gimp_object_get_name (editor->drawable);
 
   gimp_editor_set_name (GIMP_EDITOR (editor), name);
 }
@@ -513,28 +651,31 @@ gimp_histogram_editor_info_update (GimpHistogramEditor *editor)
 
   if (hist)
     {
+      gint    n_bins;
       gdouble pixels;
       gdouble count;
       gchar   text[12];
 
-      pixels = gimp_histogram_get_count (hist, view->channel, 0, 255);
+      n_bins = gimp_histogram_n_bins (hist);
+
+      pixels = gimp_histogram_get_count (hist, view->channel, 0, n_bins - 1);
       count  = gimp_histogram_get_count (hist, view->channel,
                                          view->start, view->end);
 
-      g_snprintf (text, sizeof (text), "%.1f",
+      g_snprintf (text, sizeof (text), "%.3f",
                   gimp_histogram_get_mean (hist, view->channel,
                                            view->start, view->end));
       gtk_label_set_text (GTK_LABEL (editor->labels[0]), text);
 
-      g_snprintf (text, sizeof (text), "%.1f",
+      g_snprintf (text, sizeof (text), "%.3f",
                   gimp_histogram_get_std_dev (hist, view->channel,
                                               view->start, view->end));
       gtk_label_set_text (GTK_LABEL (editor->labels[1]), text);
 
-      g_snprintf (text, sizeof (text), "%.1f",
-                  (gdouble) gimp_histogram_get_median  (hist, view->channel,
-                                                        view->start,
-                                                        view->end));
+      g_snprintf (text, sizeof (text), "%.3f",
+                  gimp_histogram_get_median  (hist, view->channel,
+                                              view->start,
+                                              view->end));
       gtk_label_set_text (GTK_LABEL (editor->labels[2]), text);
 
       g_snprintf (text, sizeof (text), "%d", (gint) pixels);
