@@ -1,9 +1,9 @@
 /* GIMP - The GNU Image Manipulation Program
  * Copyright (C) 1995 Spencer Kimball and Peter Mattis
  *
- * This program is free software; you can redistribute it and/or modify
+ * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -12,14 +12,14 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
 
 #include <string.h>
 
+#include <gegl.h>
 #include <gtk/gtk.h>
 
 #include "libgimpbase/gimpbase.h"
@@ -36,10 +36,7 @@
 #include "core/gimpprogress.h"
 #include "core/gimpsubprogress.h"
 
-#include "plug-in/gimppluginmanager.h"
-
-#include "file/file-procedure.h"
-#include "file/file-utils.h"
+#include "plug-in/gimppluginmanager-file.h"
 
 #include "gimpfiledialog.h" /* eek */
 #include "gimpthumbbox.h"
@@ -57,15 +54,13 @@ static void     gimp_thumb_box_progress_iface_init (GimpProgressInterface *iface
 static void     gimp_thumb_box_dispose            (GObject           *object);
 static void     gimp_thumb_box_finalize           (GObject           *object);
 
-static void     gimp_thumb_box_destroy            (GtkObject         *object);
-
 static void     gimp_thumb_box_style_set          (GtkWidget         *widget,
                                                    GtkStyle          *prev_style);
 
 static GimpProgress *
                 gimp_thumb_box_progress_start     (GimpProgress      *progress,
-                                                   const gchar       *message,
-                                                   gboolean           cancelable);
+                                                   gboolean           cancellable,
+                                                   const gchar       *message);
 static void     gimp_thumb_box_progress_end       (GimpProgress      *progress);
 static gboolean gimp_thumb_box_progress_is_active (GimpProgress      *progress);
 static void     gimp_thumb_box_progress_set_value (GimpProgress      *progress,
@@ -93,7 +88,7 @@ static void gimp_thumb_box_thumb_state_notify     (GimpThumbnail     *thumb,
 static void gimp_thumb_box_create_thumbnails      (GimpThumbBox      *box,
                                                    gboolean           force);
 static void gimp_thumb_box_create_thumbnail       (GimpThumbBox      *box,
-                                                   const gchar       *uri,
+                                                   GFile             *file,
                                                    GimpThumbnailSize  size,
                                                    gboolean           force,
                                                    GimpProgress      *progress);
@@ -110,15 +105,12 @@ G_DEFINE_TYPE_WITH_CODE (GimpThumbBox, gimp_thumb_box, GTK_TYPE_FRAME,
 static void
 gimp_thumb_box_class_init (GimpThumbBoxClass *klass)
 {
-  GObjectClass   *object_class     = G_OBJECT_CLASS (klass);
-  GtkObjectClass *gtk_object_class = GTK_OBJECT_CLASS (klass);
-  GtkWidgetClass *widget_class     = GTK_WIDGET_CLASS (klass);
+  GObjectClass   *object_class = G_OBJECT_CLASS (klass);
+  GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
 
   object_class->dispose     = gimp_thumb_box_dispose;
   object_class->finalize    = gimp_thumb_box_finalize;
 
-  gtk_object_class->destroy = gimp_thumb_box_destroy;
-
   widget_class->style_set   = gimp_thumb_box_style_set;
 }
 
@@ -154,6 +146,8 @@ gimp_thumb_box_dispose (GObject *object)
     }
 
   G_OBJECT_CLASS (parent_class)->dispose (object);
+
+  box->progress = NULL;
 }
 
 static void
@@ -161,7 +155,7 @@ gimp_thumb_box_finalize (GObject *object)
 {
   GimpThumbBox *box = GIMP_THUMB_BOX (object);
 
-  gimp_thumb_box_take_uris (box, NULL);
+  gimp_thumb_box_take_files (box, NULL);
 
   if (box->imagefile)
     {
@@ -172,16 +166,6 @@ gimp_thumb_box_finalize (GObject *object)
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
-static void
-gimp_thumb_box_destroy (GtkObject *object)
-{
-  GimpThumbBox *box = GIMP_THUMB_BOX (object);
-
-  GTK_OBJECT_CLASS (parent_class)->destroy (object);
-
-  box->progress = NULL;
-}
-
 static void
 gimp_thumb_box_style_set (GtkWidget *widget,
                           GtkStyle  *prev_style)
@@ -207,8 +191,8 @@ gimp_thumb_box_style_set (GtkWidget *widget,
 
 static GimpProgress *
 gimp_thumb_box_progress_start (GimpProgress *progress,
-                               const gchar  *message,
-                               gboolean      cancelable)
+                               gboolean      cancellable,
+                               const gchar  *message)
 {
   GimpThumbBox *box = GIMP_THUMB_BOX (progress);
 
@@ -228,7 +212,7 @@ gimp_thumb_box_progress_start (GimpProgress *progress,
 
       if (GIMP_IS_FILE_DIALOG (toplevel))
         gtk_dialog_set_response_sensitive (GTK_DIALOG (toplevel),
-                                           GTK_RESPONSE_CANCEL, cancelable);
+                                           GTK_RESPONSE_CANCEL, cancellable);
 
       return progress;
     }
@@ -342,16 +326,15 @@ gimp_thumb_box_new (GimpContext *context)
                     box);
 
   str = g_strdup_printf (_("Click to update preview\n"
-                           "%s%sClick to force update even "
+                           "%s-Click to force update even "
                            "if preview is up-to-date"),
-                         gimp_get_mod_string (GDK_CONTROL_MASK),
-                         gimp_get_mod_separator ());
+                         gimp_get_mod_string (gimp_get_toggle_behavior_mask ()));
 
   gimp_help_set_help_data (ebox, str, NULL);
 
   g_free (str);
 
-  vbox = gtk_vbox_new (FALSE, 0);
+  vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
   gtk_container_add (GTK_CONTAINER (ebox), vbox);
   gtk_widget_show (vbox);
 
@@ -360,7 +343,7 @@ gimp_thumb_box_new (GimpContext *context)
   gtk_widget_show (button);
 
   label = gtk_label_new_with_mnemonic (_("Pr_eview"));
-  gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
+  gtk_label_set_xalign (GTK_LABEL (label), 0.0);
   gtk_container_add (GTK_CONTAINER (button), label);
   gtk_widget_show (label);
 
@@ -377,12 +360,13 @@ gimp_thumb_box_new (GimpContext *context)
                     G_CALLBACK (gtk_true),
                     NULL);
 
-  vbox2 = gtk_vbox_new (FALSE, 6);
+  vbox2 = gtk_box_new (GTK_ORIENTATION_VERTICAL, 6);
   gtk_container_set_border_width (GTK_CONTAINER (vbox2), 2);
-  gtk_container_add (GTK_CONTAINER (vbox), vbox2);
+  gtk_box_pack_start (GTK_BOX (vbox), vbox2, TRUE, TRUE, 0);
   gtk_widget_show (vbox2);
 
-  hbox = gtk_hbox_new (TRUE, 0);
+  hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
+  gtk_box_set_homogeneous (GTK_BOX (hbox), TRUE);
   gtk_box_pack_start (GTK_BOX (vbox2), hbox, FALSE, FALSE, 0);
   gtk_widget_show (hbox);
 
@@ -392,7 +376,8 @@ gimp_thumb_box_new (GimpContext *context)
                     G_CALLBACK (gimp_thumb_box_imagefile_info_changed),
                     box);
 
-  g_signal_connect (box->imagefile->thumbnail, "notify::thumb-state",
+  g_signal_connect (gimp_imagefile_get_thumbnail (box->imagefile),
+                    "notify::thumb-state",
                     G_CALLBACK (gimp_thumb_box_thumb_state_notify),
                     box);
 
@@ -424,7 +409,6 @@ gimp_thumb_box_new (GimpContext *context)
   gtk_widget_show (box->filename);
 
   box->info = gtk_label_new (" \n \n \n ");
-  gtk_misc_set_alignment (GTK_MISC (box->info), 0.5, 0.0);
   gtk_label_set_justify (GTK_LABEL (box->info), GTK_JUSTIFY_CENTER);
   gtk_label_set_line_wrap (GTK_LABEL (box->info), TRUE);
   gimp_label_set_attributes (GTK_LABEL (box->info),
@@ -456,10 +440,11 @@ gimp_thumb_box_new (GimpContext *context)
 }
 
 void
-gimp_thumb_box_take_uri (GimpThumbBox *box,
-                         gchar        *uri)
+gimp_thumb_box_take_file (GimpThumbBox *box,
+                          GFile        *file)
 {
   g_return_if_fail (GIMP_IS_THUMB_BOX (box));
+  g_return_if_fail (file == NULL || G_IS_FILE (file));
 
   if (box->idle_id)
     {
@@ -467,12 +452,11 @@ gimp_thumb_box_take_uri (GimpThumbBox *box,
       box->idle_id = 0;
     }
 
-  gimp_object_take_name (GIMP_OBJECT (box->imagefile), uri);
+  gimp_imagefile_set_file (box->imagefile, file);
 
-  if (uri)
+  if (file)
     {
-      gchar *basename = file_utils_uri_display_basename (uri);
-
+      gchar *basename = g_path_get_basename (gimp_file_get_utf8_name (file));
       gtk_label_set_text (GTK_LABEL (box->filename), basename);
       g_free (basename);
     }
@@ -481,24 +465,23 @@ gimp_thumb_box_take_uri (GimpThumbBox *box,
       gtk_label_set_text (GTK_LABEL (box->filename), _("No selection"));
     }
 
-  gtk_widget_set_sensitive (GTK_WIDGET (box), uri != NULL);
+  gtk_widget_set_sensitive (GTK_WIDGET (box), file != NULL);
   gimp_imagefile_update (box->imagefile);
 }
 
 void
-gimp_thumb_box_take_uris (GimpThumbBox *box,
-                          GSList       *uris)
+gimp_thumb_box_take_files (GimpThumbBox *box,
+                           GSList       *files)
 {
   g_return_if_fail (GIMP_IS_THUMB_BOX (box));
 
-  if (box->uris)
+  if (box->files)
     {
-      g_slist_foreach (box->uris, (GFunc) g_free, NULL);
-      g_slist_free (box->uris);
-      box->uris = NULL;
+      g_slist_free_full (box->files, (GDestroyNotify) g_object_unref);
+      box->files = NULL;
     }
 
-  box->uris = uris;
+  box->files = files;
 }
 
 
@@ -520,7 +503,20 @@ gimp_thumb_box_thumbnail_clicked (GtkWidget       *widget,
                                   GimpThumbBox    *box)
 {
   gimp_thumb_box_create_thumbnails (box,
-                                    (state & GDK_CONTROL_MASK) ? TRUE : FALSE);
+                                    (state & gimp_get_toggle_behavior_mask ()) ?
+                                    TRUE : FALSE);
+}
+
+static void
+this_is_ugly (GtkWidget     *widget,
+              GtkAllocation *allocation,
+              gpointer       data)
+{
+  gtk_widget_queue_resize (widget);
+
+  g_signal_handlers_disconnect_by_func (widget,
+                                        this_is_ugly,
+                                        data);
 }
 
 static void
@@ -529,6 +525,10 @@ gimp_thumb_box_imagefile_info_changed (GimpImagefile *imagefile,
 {
   gtk_label_set_text (GTK_LABEL (box->info),
                       gimp_imagefile_get_desc_string (imagefile));
+
+  g_signal_connect_after (box->info, "size-allocate",
+                          G_CALLBACK (this_is_ugly),
+                          "this too");
 }
 
 static void
@@ -561,12 +561,12 @@ static void
 gimp_thumb_box_create_thumbnails (GimpThumbBox *box,
                                   gboolean      force)
 {
-  Gimp           *gimp     = box->imagefile->gimp;
+  Gimp           *gimp     = box->context->gimp;
   GimpProgress   *progress = GIMP_PROGRESS (box);
   GimpFileDialog *dialog   = NULL;
   GtkWidget      *toplevel;
   GSList         *list;
-  gint            n_uris;
+  gint            n_files;
   gint            i;
 
   if (gimp->config->thumbnail_size == GIMP_THUMBNAIL_SIZE_NONE)
@@ -584,29 +584,29 @@ gimp_thumb_box_create_thumbnails (GimpThumbBox *box,
   else
     gtk_widget_set_sensitive (toplevel, FALSE);
 
-  if (box->uris)
+  if (box->files)
     {
       gtk_widget_hide (box->info);
       gtk_widget_show (box->progress);
     }
 
-  n_uris = g_slist_length (box->uris);
+  n_files = g_slist_length (box->files);
 
-  if (n_uris > 1)
+  if (n_files > 1)
     {
       gchar *str;
 
-      gimp_progress_start (GIMP_PROGRESS (box), "", TRUE);
+      gimp_progress_start (GIMP_PROGRESS (box), TRUE, "%s", "");
 
       progress = gimp_sub_progress_new (GIMP_PROGRESS (box));
 
-      gimp_sub_progress_set_step (GIMP_SUB_PROGRESS (progress), 0, n_uris);
+      gimp_sub_progress_set_step (GIMP_SUB_PROGRESS (progress), 0, n_files);
 
-      for (list = box->uris->next, i = 1;
+      for (list = box->files->next, i = 1;
            list;
            list = g_slist_next (list), i++)
         {
-          str = g_strdup_printf (_("Thumbnail %d of %d"), i, n_uris);
+          str = g_strdup_printf (_("Thumbnail %d of %d"), i, n_files);
           gtk_progress_bar_set_text (GTK_PROGRESS_BAR (box->progress), str);
           g_free (str);
 
@@ -624,10 +624,10 @@ gimp_thumb_box_create_thumbnails (GimpThumbBox *box,
           if (dialog && dialog->canceled)
             goto canceled;
 
-          gimp_sub_progress_set_step (GIMP_SUB_PROGRESS (progress), i, n_uris);
+          gimp_sub_progress_set_step (GIMP_SUB_PROGRESS (progress), i, n_files);
         }
 
-      str = g_strdup_printf (_("Thumbnail %d of %d"), n_uris, n_uris);
+      str = g_strdup_printf (_("Thumbnail %d of %d"), n_files, n_files);
       gtk_progress_bar_set_text (GTK_PROGRESS_BAR (box->progress), str);
       g_free (str);
 
@@ -637,10 +637,10 @@ gimp_thumb_box_create_thumbnails (GimpThumbBox *box,
         gtk_main_iteration ();
     }
 
-  if (box->uris)
+  if (box->files)
     {
       gimp_thumb_box_create_thumbnail (box,
-                                       box->uris->data,
+                                       box->files->data,
                                        gimp->config->thumbnail_size,
                                        force,
                                        progress);
@@ -650,7 +650,7 @@ gimp_thumb_box_create_thumbnails (GimpThumbBox *box,
 
  canceled:
 
-  if (n_uris > 1)
+  if (n_files > 1)
     {
       g_object_unref (progress);
 
@@ -658,7 +658,7 @@ gimp_thumb_box_create_thumbnails (GimpThumbBox *box,
       gtk_progress_bar_set_text (GTK_PROGRESS_BAR (box->progress), "");
     }
 
-  if (box->uris)
+  if (box->files)
     {
       gtk_widget_hide (box->progress);
       gtk_widget_show (box->info);
@@ -674,50 +674,44 @@ gimp_thumb_box_create_thumbnails (GimpThumbBox *box,
 
 static void
 gimp_thumb_box_create_thumbnail (GimpThumbBox      *box,
-                                 const gchar       *uri,
+                                 GFile             *file,
                                  GimpThumbnailSize  size,
                                  gboolean           force,
                                  GimpProgress      *progress)
 {
-  gchar         *filename = file_utils_filename_from_uri (uri);
-  GimpThumbnail *thumb;
+  GimpThumbnail *thumb = gimp_imagefile_get_thumbnail (box->imagefile);
   gchar         *basename;
 
-  if (filename)
-    {
-      gboolean regular = g_file_test (filename, G_FILE_TEST_IS_REGULAR);
-
-      g_free (filename);
-
-      if (! regular)
-        return;
-    }
-
-  thumb = box->imagefile->thumbnail;
-
-  basename = file_utils_uri_display_basename (uri);
+  basename = g_path_get_basename (gimp_file_get_utf8_name (file));
   gtk_label_set_text (GTK_LABEL (box->filename), basename);
   g_free (basename);
 
-  gimp_object_set_name (GIMP_OBJECT (box->imagefile), uri);
+  gimp_imagefile_set_file (box->imagefile, file);
 
   if (force ||
-      (gimp_thumbnail_peek_thumb (thumb, size) < GIMP_THUMB_STATE_FAILED &&
+      (gimp_thumbnail_peek_thumb (thumb, (GimpThumbSize) size) < GIMP_THUMB_STATE_FAILED &&
        ! gimp_thumbnail_has_failed (thumb)))
     {
-      gimp_imagefile_create_thumbnail (box->imagefile, box->context,
-                                       progress,
-                                       size,
-                                       !force);
+      GError *error = NULL;
+
+      if (! gimp_imagefile_create_thumbnail (box->imagefile, box->context,
+                                             progress,
+                                             size, ! force, &error))
+        {
+          gimp_message_literal (box->context->gimp,
+                                G_OBJECT (progress), GIMP_MESSAGE_ERROR,
+                                error->message);
+          g_clear_error (&error);
+        }
     }
 }
 
 static gboolean
 gimp_thumb_box_auto_thumbnail (GimpThumbBox *box)
 {
-  Gimp          *gimp  = box->imagefile->gimp;
-  GimpThumbnail *thumb = box->imagefile->thumbnail;
-  const gchar   *uri   = gimp_object_get_name (GIMP_OBJECT (box->imagefile));
+  Gimp          *gimp  = box->context->gimp;
+  GimpThumbnail *thumb = gimp_imagefile_get_thumbnail (box->imagefile);
+  GFile         *file  = gimp_imagefile_get_file (box->imagefile);
 
   box->idle_id = 0;
 
@@ -730,15 +724,16 @@ gimp_thumb_box_auto_thumbnail (GimpThumbBox *box)
     case GIMP_THUMB_STATE_OLD:
       if (thumb->image_filesize < gimp->config->thumbnail_filesize_limit &&
           ! gimp_thumbnail_has_failed (thumb)                            &&
-          file_procedure_find_by_extension (gimp->plug_in_manager->load_procs,
-                                            uri))
+          gimp_plug_in_manager_file_procedure_find_by_extension (gimp->plug_in_manager,
+                                                                 GIMP_FILE_PROCEDURE_GROUP_OPEN,
+                                                                 file))
         {
           if (thumb->image_filesize > 0)
             {
               gchar *size;
               gchar *text;
 
-              size = g_format_size_for_display (thumb->image_filesize);
+              size = g_format_size (thumb->image_filesize);
               text = g_strdup_printf ("%s\n%s",
                                       size, _("Creating preview..."));
 
