@@ -44,7 +44,8 @@
 enum
 {
   PROP_0,
-  PROP_ELLIPSIZE
+  PROP_ELLIPSIZE,
+  PROP_LABEL
 };
 
 
@@ -53,6 +54,10 @@ typedef struct
   GtkCellRenderer        *pixbuf_renderer;
   GtkCellRenderer        *text_renderer;
 
+  PangoEllipsizeMode      ellipsize;
+  gchar                  *label;
+  GtkCellRenderer        *label_renderer;
+
   GimpIntSensitivityFunc  sensitivity_func;
   gpointer                sensitivity_data;
   GDestroyNotify          sensitivity_destroy;
@@ -72,6 +77,7 @@ static void  gimp_int_combo_box_get_property (GObject         *object,
                                               GValue          *value,
                                               GParamSpec      *pspec);
 
+static void  gimp_int_combo_box_create_cells (GimpIntComboBox *combo_box);
 static void  gimp_int_combo_box_data_func    (GtkCellLayout   *layout,
                                               GtkCellRenderer *cell,
                                               GtkTreeModel    *model,
@@ -89,9 +95,9 @@ gimp_int_combo_box_class_init (GimpIntComboBoxClass *klass)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
+  object_class->finalize     = gimp_int_combo_box_finalize;
   object_class->set_property = gimp_int_combo_box_set_property;
   object_class->get_property = gimp_int_combo_box_get_property;
-  object_class->finalize     = gimp_int_combo_box_finalize;
 
   /**
    * GimpIntComboBox:ellipsize:
@@ -100,48 +106,47 @@ gimp_int_combo_box_class_init (GimpIntComboBoxClass *klass)
    * if the cell renderer does not have enough room to display the
    * entire string.
    *
-   * Since: GIMP 2.4
+   * Since: 2.4
    */
-  g_object_class_install_property (object_class,
-                                   PROP_ELLIPSIZE,
-                                   g_param_spec_enum ("ellipsize", NULL, NULL,
+  g_object_class_install_property (object_class, PROP_ELLIPSIZE,
+                                   g_param_spec_enum ("ellipsize",
+                                                      "Ellipsize",
+                                                      "Ellipsize mode for the used text cell renderer",
                                                       PANGO_TYPE_ELLIPSIZE_MODE,
                                                       PANGO_ELLIPSIZE_NONE,
                                                       GIMP_PARAM_READWRITE));
 
+  /**
+   * GimpIntComboBox:label:
+   *
+   * Sets a label on the combo-box, see gimp_int_combo_box_set_label().
+   *
+   * Since: 2.10
+   */
+  g_object_class_install_property (object_class, PROP_LABEL,
+                                   g_param_spec_string ("label",
+                                                        "Label",
+                                                        "An optional label to be displayed",
+                                                        NULL,
+                                                        GIMP_PARAM_READWRITE));
+
   g_type_class_add_private (object_class, sizeof (GimpIntComboBoxPrivate));
 }
 
 static void
 gimp_int_combo_box_init (GimpIntComboBox *combo_box)
 {
-  GimpIntComboBoxPrivate *priv;
-  GtkListStore           *store;
-  GtkCellRenderer        *cell;
+  GtkListStore *store;
 
   combo_box->priv = G_TYPE_INSTANCE_GET_PRIVATE (combo_box,
                                                  GIMP_TYPE_INT_COMBO_BOX,
                                                  GimpIntComboBoxPrivate);
 
-  priv = GIMP_INT_COMBO_BOX_GET_PRIVATE (combo_box);
-
   store = gimp_int_store_new ();
   gtk_combo_box_set_model (GTK_COMBO_BOX (combo_box), GTK_TREE_MODEL (store));
   g_object_unref (store);
 
-  priv->pixbuf_renderer = cell = gtk_cell_renderer_pixbuf_new ();
-  gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (combo_box), cell, FALSE);
-  gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (combo_box), cell,
-                                  "stock-id", GIMP_INT_STORE_STOCK_ID,
-                                  "pixbuf",   GIMP_INT_STORE_PIXBUF,
-                                  NULL);
-
-  priv->text_renderer = cell = gtk_cell_renderer_text_new ();
-
-  gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (combo_box), cell, TRUE);
-  gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (combo_box), cell,
-                                  "text", GIMP_INT_STORE_LABEL,
-                                  NULL);
+  gimp_int_combo_box_create_cells (GIMP_INT_COMBO_BOX (combo_box));
 }
 
 static void
@@ -149,6 +154,12 @@ gimp_int_combo_box_finalize (GObject *object)
 {
   GimpIntComboBoxPrivate *priv = GIMP_INT_COMBO_BOX_GET_PRIVATE (object);
 
+  if (priv->label)
+    {
+      g_free (priv->label);
+      priv->label = NULL;
+    }
+
   if (priv->sensitivity_destroy)
     {
       GDestroyNotify d = priv->sensitivity_destroy;
@@ -171,9 +182,15 @@ gimp_int_combo_box_set_property (GObject      *object,
   switch (property_id)
     {
     case PROP_ELLIPSIZE:
+      priv->ellipsize = g_value_get_enum (value);
       g_object_set_property (G_OBJECT (priv->text_renderer),
                              pspec->name, value);
       break;
+    case PROP_LABEL:
+      gimp_int_combo_box_set_label (GIMP_INT_COMBO_BOX (object),
+                                    g_value_get_string (value));
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
       break;
@@ -191,9 +208,12 @@ gimp_int_combo_box_get_property (GObject    *object,
   switch (property_id)
     {
     case PROP_ELLIPSIZE:
-      g_object_get_property (G_OBJECT (priv->text_renderer),
-                             pspec->name, value);
+      g_value_set_enum (value, priv->ellipsize);
       break;
+    case PROP_LABEL:
+      g_value_set_string (value, priv->label);
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
       break;
@@ -216,7 +236,7 @@ gimp_int_combo_box_get_property (GObject    *object,
  *
  * Return value: a new #GimpIntComboBox.
  *
- * Since: GIMP 2.2
+ * Since: 2.2
  **/
 GtkWidget *
 gimp_int_combo_box_new (const gchar *first_label,
@@ -246,7 +266,7 @@ gimp_int_combo_box_new (const gchar *first_label,
  *
  * Return value: a new #GimpIntComboBox.
  *
- * Since: GIMP 2.2
+ * Since: 2.2
  **/
 GtkWidget *
 gimp_int_combo_box_new_valist (const gchar *first_label,
@@ -288,7 +308,7 @@ gimp_int_combo_box_new_valist (const gchar *first_label,
  *
  * Return value: a new #GimpIntComboBox.
  *
- * Since: GIMP 2.2
+ * Since: 2.2
  **/
 GtkWidget *
 gimp_int_combo_box_new_array (gint         n_values,
@@ -333,7 +353,7 @@ gimp_int_combo_box_new_array (gint         n_values,
  *
  * The column number must be taken from the enum #GimpIntStoreColumns.
  *
- * Since: GIMP 2.2
+ * Since: 2.2
  **/
 void
 gimp_int_combo_box_prepend (GimpIntComboBox *combo_box,
@@ -366,7 +386,7 @@ gimp_int_combo_box_prepend (GimpIntComboBox *combo_box,
  *
  * The column number must be taken from the enum #GimpIntStoreColumns.
  *
- * Since: GIMP 2.2
+ * Since: 2.2
  **/
 void
 gimp_int_combo_box_append (GimpIntComboBox *combo_box,
@@ -399,7 +419,7 @@ gimp_int_combo_box_append (GimpIntComboBox *combo_box,
  * Return value: %TRUE on success or %FALSE if there was no item for
  *               this value.
  *
- * Since: GIMP 2.2
+ * Since: 2.2
  **/
 gboolean
 gimp_int_combo_box_set_active (GimpIntComboBox *combo_box,
@@ -431,7 +451,7 @@ gimp_int_combo_box_set_active (GimpIntComboBox *combo_box,
  * Return value: %TRUE if @value has been set or %FALSE if no item was
  *               active.
  *
- * Since: GIMP 2.2
+ * Since: 2.2
  **/
 gboolean
 gimp_int_combo_box_get_active (GimpIntComboBox *combo_box,
@@ -454,6 +474,72 @@ gimp_int_combo_box_get_active (GimpIntComboBox *combo_box,
   return FALSE;
 }
 
+/**
+ * gimp_int_combo_box_set_active_by_user_data:
+ * @combo_box: a #GimpIntComboBox
+ * @user_data: an integer value
+ *
+ * Looks up the item that has the given @user_data and makes it the
+ * selected item in the @combo_box.
+ *
+ * Return value: %TRUE on success or %FALSE if there was no item for
+ *               this user-data.
+ *
+ * Since: 2.10
+ **/
+gboolean
+gimp_int_combo_box_set_active_by_user_data (GimpIntComboBox *combo_box,
+                                            gpointer         user_data)
+{
+  GtkTreeModel *model;
+  GtkTreeIter   iter;
+
+  g_return_val_if_fail (GIMP_IS_INT_COMBO_BOX (combo_box), FALSE);
+
+  model = gtk_combo_box_get_model (GTK_COMBO_BOX (combo_box));
+
+  if (gimp_int_store_lookup_by_user_data (model, user_data, &iter))
+    {
+      gtk_combo_box_set_active_iter (GTK_COMBO_BOX (combo_box), &iter);
+      return TRUE;
+    }
+
+  return FALSE;
+}
+
+/**
+ * gimp_int_combo_box_get_active_user_data:
+ * @combo_box: a #GimpIntComboBox
+ * @user_data: return location for the gpointer value
+ *
+ * Retrieves the user-data of the selected (active) item in the @combo_box.
+ *
+ * Return value: %TRUE if @user_data has been set or %FALSE if no item was
+ *               active.
+ *
+ * Since: 2.10
+ **/
+gboolean
+gimp_int_combo_box_get_active_user_data (GimpIntComboBox *combo_box,
+                                         gpointer        *user_data)
+{
+  GtkTreeIter  iter;
+
+  g_return_val_if_fail (GIMP_IS_INT_COMBO_BOX (combo_box), FALSE);
+  g_return_val_if_fail (user_data != NULL, FALSE);
+
+  if (gtk_combo_box_get_active_iter (GTK_COMBO_BOX (combo_box), &iter))
+    {
+      gtk_tree_model_get (gtk_combo_box_get_model (GTK_COMBO_BOX (combo_box)),
+                          &iter,
+                          GIMP_INT_STORE_USER_DATA, user_data,
+                          -1);
+      return TRUE;
+    }
+
+  return FALSE;
+}
+
 /**
  * gimp_int_combo_box_connect:
  * @combo_box: a #GimpIntComboBox
@@ -461,7 +547,7 @@ gimp_int_combo_box_get_active (GimpIntComboBox *combo_box,
  * @callback:  a callback to connect to the @combo_box's "changed" signal
  * @data:      a pointer passed as data to g_signal_connect()
  *
- * A convenience function that sets the inital @value of a
+ * A convenience function that sets the initial @value of a
  * #GimpIntComboBox and connects @callback to the "changed"
  * signal.
  *
@@ -473,7 +559,7 @@ gimp_int_combo_box_get_active (GimpIntComboBox *combo_box,
  *
  * Return value: the signal handler ID as returned by g_signal_connect()
  *
- * Since: GIMP 2.2
+ * Since: 2.2
  **/
 gulong
 gimp_int_combo_box_connect (GimpIntComboBox *combo_box,
@@ -494,6 +580,74 @@ gimp_int_combo_box_connect (GimpIntComboBox *combo_box,
   return handler;
 }
 
+/**
+ * gimp_int_combo_box_set_label:
+ * @combo_box: a #GimpIntComboBox
+ * @label:     a string to be shown as label
+ *
+ * Sets a caption on the @combo_box that will be displayed
+ * left-aligned inside the box. When a label is set, the remaining
+ * contents of the box will be right-aligned. This is useful for
+ * places where screen estate is rare, like in tool options.
+ *
+ * Since: 2.10
+ **/
+void
+gimp_int_combo_box_set_label (GimpIntComboBox *combo_box,
+                              const gchar     *label)
+{
+  GimpIntComboBoxPrivate *priv;
+
+  g_return_if_fail (GIMP_IS_INT_COMBO_BOX (combo_box));
+
+  priv = GIMP_INT_COMBO_BOX_GET_PRIVATE (combo_box);
+
+  if (label == priv->label)
+    return;
+
+  if (priv->label)
+    {
+      g_free (priv->label);
+      priv->label = NULL;
+
+      g_signal_handlers_disconnect_by_func (combo_box,
+                                            gimp_int_combo_box_create_cells,
+                                            NULL);
+    }
+
+  if (label)
+    {
+      priv->label = g_strdup (label);
+
+      g_signal_connect (combo_box, "notify::popup-shown",
+                        G_CALLBACK (gimp_int_combo_box_create_cells),
+                        NULL);
+    }
+
+  gimp_int_combo_box_create_cells (combo_box);
+
+  g_object_notify (G_OBJECT (combo_box), "label");
+}
+
+/**
+ * gimp_int_combo_box_get_label:
+ * @combo_box: a #GimpIntComboBox
+ *
+ * Returns the label previously set with gimp_int_combo_box_set_label(),
+ * or %NULL,
+ *
+ * Return value: the @combo_box' label.
+ *
+ * Since: 2.10
+ **/
+const gchar *
+gimp_int_combo_box_get_label (GimpIntComboBox *combo_box)
+{
+  g_return_val_if_fail (GIMP_IS_INT_COMBO_BOX (combo_box), NULL);
+
+  return GIMP_INT_COMBO_BOX_GET_PRIVATE (combo_box)->label;
+}
+
 /**
  * gimp_int_combo_box_set_sensitivity:
  * @combo_box: a #GimpIntComboBox
@@ -508,7 +662,7 @@ gimp_int_combo_box_connect (GimpIntComboBox *combo_box,
  * Calling gtk_widget_queue_draw() on the @combo_box will cause the
  * sensitivity to be updated.
  *
- * Since: GIMP 2.4
+ * Since: 2.4
  **/
 void
 gimp_int_combo_box_set_sensitivity (GimpIntComboBox        *combo_box,
@@ -548,6 +702,120 @@ gimp_int_combo_box_set_sensitivity (GimpIntComboBox        *combo_box,
 }
 
 
+/*  private functions  */
+
+static void
+queue_resize_cell_view (GtkContainer *container)
+{
+  GList *children = gtk_container_get_children (container);
+  GList *list;
+
+  for (list = children; list; list = g_list_next (list))
+    {
+      if (GTK_IS_CELL_VIEW (list->data))
+        {
+          gtk_widget_queue_resize (list->data);
+          break;
+        }
+      else if (GTK_IS_CONTAINER (list->data))
+        {
+          queue_resize_cell_view (list->data);
+        }
+    }
+
+  g_list_free (children);
+}
+
+static void
+gimp_int_combo_box_create_cells (GimpIntComboBox *combo_box)
+{
+  GimpIntComboBoxPrivate *priv = GIMP_INT_COMBO_BOX_GET_PRIVATE (combo_box);
+  gboolean                shown;
+
+  g_object_get (combo_box, "popup-shown", &shown, NULL);
+
+  gtk_cell_layout_clear (GTK_CELL_LAYOUT (combo_box));
+
+  priv->pixbuf_renderer = gtk_cell_renderer_pixbuf_new ();
+  g_object_set (priv->pixbuf_renderer,
+                "xpad", 2,
+                NULL);
+
+  priv->text_renderer = gtk_cell_renderer_text_new ();
+
+  if (! shown)
+    g_object_set (priv->text_renderer,
+                  "ellipsize", priv->ellipsize,
+                  NULL);
+
+  if (priv->label && ! shown)
+    {
+      priv->label_renderer = gtk_cell_renderer_text_new ();
+      g_object_set (priv->label_renderer,
+                    "text", priv->label,
+                    NULL);
+
+      gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (combo_box),
+                                  priv->label_renderer, FALSE);
+
+      gtk_cell_layout_pack_end (GTK_CELL_LAYOUT (combo_box),
+                                priv->pixbuf_renderer, FALSE);
+      gtk_cell_layout_pack_end (GTK_CELL_LAYOUT (combo_box),
+                                priv->text_renderer, TRUE);
+
+      g_object_set (priv->text_renderer,
+                    "xalign", 1.0,
+                    NULL);
+    }
+  else
+    {
+      gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (combo_box),
+                                  priv->pixbuf_renderer, FALSE);
+      gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (combo_box),
+                                  priv->text_renderer, TRUE);
+    }
+
+  gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (combo_box),
+                                  priv->pixbuf_renderer,
+                                  "icon-name", GIMP_INT_STORE_ICON_NAME,
+                                  "pixbuf",    GIMP_INT_STORE_PIXBUF,
+                                  NULL);
+  gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (combo_box),
+                                  priv->text_renderer,
+                                  "text", GIMP_INT_STORE_LABEL,
+                                  NULL);
+
+  if (priv->sensitivity_func)
+    {
+      gtk_cell_layout_set_cell_data_func (GTK_CELL_LAYOUT (combo_box),
+                                          priv->pixbuf_renderer,
+                                          gimp_int_combo_box_data_func,
+                                          priv, NULL);
+
+      gtk_cell_layout_set_cell_data_func (GTK_CELL_LAYOUT (combo_box),
+                                          priv->text_renderer,
+                                          gimp_int_combo_box_data_func,
+                                          priv, NULL);
+    }
+
+  /* HACK: GtkCellView doesn't invalidate itself when stuff is
+   * added/removed, work around this bug until GTK+ 2.24.19
+   */
+  if (gtk_check_version (2, 24, 19))
+    {
+      GList *attached_menus;
+
+      queue_resize_cell_view (GTK_CONTAINER (combo_box));
+
+      /* HACK HACK HACK OMG */
+      attached_menus = g_object_get_data (G_OBJECT (combo_box),
+                                          "gtk-attached-menus");
+
+      for (; attached_menus; attached_menus = g_list_next (attached_menus))
+        queue_resize_cell_view (attached_menus->data);
+    }
+}
+
 static void
 gimp_int_combo_box_data_func (GtkCellLayout   *layout,
                               GtkCellRenderer *cell,
