@@ -1,5 +1,5 @@
 /*
- * pcx.c GIMP plug-in for loading & saving PCX files
+ * pcx.c GIMP plug-in for loading & exporting PCX files
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -37,59 +37,70 @@
 #define PLUG_IN_BINARY "file-pcx"
 #define PLUG_IN_ROLE   "gimp-file-pcx"
 
+
 /* Declare local functions.  */
 
-static void   query      (void);
-static void   run        (const gchar      *name,
-                          gint              nparams,
-                          const GimpParam  *param,
-                          gint             *nreturn_vals,
-                          GimpParam       **return_vals);
+static void   query            (void);
+static void   run              (const gchar      *name,
+                                gint              nparams,
+                                const GimpParam  *param,
+                                gint             *nreturn_vals,
+                                GimpParam       **return_vals);
 
-static gint32 load_image (const gchar      *filename,
-                          GError          **error);
+static gint32 load_image       (const gchar      *filename,
+                                GError          **error);
 
-static void   load_1     (FILE             *fp,
-                          gint              width,
-                          gint              height,
-                          guchar           *buffer,
-                          guint16           bytes);
-static void   load_4     (FILE             *fp,
-                          gint              width,
-                          gint              height,
-                          guchar           *buffer,
-                          guint16           bytes);
-static void   load_8     (FILE             *fp,
-                          gint              width,
-                          gint              height,
-                          guchar           *buffer,
-                          guint16           bytes);
-static void   load_24    (FILE             *fp,
-                          gint              width,
-                          gint              height,
-                          guchar           *buffer,
-                          guint16           bytes);
-static void   readline   (FILE             *fp,
-                          guchar           *buffer,
-                          gint              bytes);
-
-static gint   save_image (const gchar      *filename,
-                          gint32            image,
-                          gint32            layer,
-                          GError          **error);
-
-static void   save_8     (FILE             *fp,
-                          gint              width,
-                          gint              height,
-                          const guchar     *buffer);
-static void   save_24    (FILE             *fp,
-                          gint              width,
-                          gint              height,
-                          const guchar     *buffer);
-static void   writeline  (FILE             *fp,
-                          const guchar     *buffer,
-                          gint              bytes);
+static void   load_1           (FILE             *fp,
+                                gint              width,
+                                gint              height,
+                                guchar           *buf,
+                                guint16           bytes);
+static void   load_4           (FILE             *fp,
+                                gint              width,
+                                gint              height,
+                                guchar           *buf,
+                                guint16           bytes);
+static void   load_sub_8       (FILE             *fp,
+                                gint              width,
+                                gint              height,
+                                gint              bpp,
+                                gint              plane,
+                                guchar           *buf,
+                                guint16           bytes);
+static void   load_8           (FILE             *fp,
+                                gint              width,
+                                gint              height,
+                                guchar           *buf,
+                                guint16           bytes);
+static void   load_24          (FILE             *fp,
+                                gint              width,
+                                gint              height,
+                                guchar           *buf,
+                                guint16           bytes);
+static void   readline         (FILE             *fp,
+                                guchar           *buf,
+                                gint              bytes);
 
+static gint   save_image       (const gchar      *filename,
+                                gint32            image,
+                                gint32            layer,
+                                GError          **error);
+static void   save_less_than_8 (FILE             *fp,
+                                gint              width,
+                                gint              height,
+                                const gint        bpp,
+                                const guchar       *buf);
+static void   save_8           (FILE             *fp,
+                                gint              width,
+                                gint              height,
+                                const guchar     *buf);
+static void   save_24          (FILE             *fp,
+                                gint              width,
+                                gint              height,
+                                const guchar     *buf);
+static void   writeline        (FILE             *fp,
+                                const guchar     *buf,
+                                gint              bytes);
 
 const GimpPlugInInfo PLUG_IN_INFO =
 {
@@ -119,8 +130,8 @@ query (void)
   {
     { GIMP_PDB_INT32,    "run-mode",     "The run mode { RUN-INTERACTIVE (0), RUN-NONINTERACTIVE (1) }" },
     { GIMP_PDB_IMAGE,    "image",        "Input image"                  },
-    { GIMP_PDB_DRAWABLE, "drawable",     "Drawable to save"             },
-    { GIMP_PDB_STRING,   "filename",     "The name of the file to save the image in" },
+    { GIMP_PDB_DRAWABLE, "drawable",     "Drawable to export"           },
+    { GIMP_PDB_STRING,   "filename",     "The name of the file to export the image in" },
     { GIMP_PDB_STRING,   "raw-filename", "The name entered"             }
   };
 
@@ -144,7 +155,7 @@ query (void)
                                     "0&,byte,10,2&,byte,1,3&,byte,>0,3,byte,<9");
 
   gimp_install_procedure (SAVE_PROC,
-                          "Saves files in ZSoft PCX file format",
+                          "Exports files in ZSoft PCX file format",
                           "FIXME: write help for pcx_save",
                           "Francisco Bustamante & Nick Lamb",
                           "Nick Lamb <njl195@zepler.org.uk>",
@@ -174,9 +185,10 @@ run (const gchar      *name,
   GimpExportReturn   export = GIMP_EXPORT_CANCEL;
   GError            *error  = NULL;
 
-  run_mode = param[0].data.d_int32;
-
   INIT_I18N ();
+  gegl_init (NULL, NULL);
+
+  run_mode = param[0].data.d_int32;
 
   *nreturn_vals = 1;
   *return_vals  = values;
@@ -211,10 +223,11 @@ run (const gchar      *name,
         case GIMP_RUN_WITH_LAST_VALS:
           gimp_ui_init (PLUG_IN_BINARY, FALSE);
 
-          export = gimp_export_image (&image_ID, &drawable_ID, NULL,
-                                      (GIMP_EXPORT_CAN_HANDLE_RGB |
-                                       GIMP_EXPORT_CAN_HANDLE_GRAY |
-                                       GIMP_EXPORT_CAN_HANDLE_INDEXED));
+          export = gimp_export_image (&image_ID, &drawable_ID, "PCX",
+                                      GIMP_EXPORT_CAN_HANDLE_RGB  |
+                                      GIMP_EXPORT_CAN_HANDLE_GRAY |
+                                      GIMP_EXPORT_CAN_HANDLE_INDEXED);
+
           if (export == GIMP_EXPORT_CANCEL)
             {
               values[0].data.d_status = GIMP_PDB_CANCEL;
@@ -269,8 +282,6 @@ run (const gchar      *name,
   values[0].data.d_status = status;
 }
 
-static const guchar mono[6]= { 0, 0, 0, 255, 255, 255 };
-
 static struct
 {
   guint8  manufacturer;
@@ -320,8 +331,8 @@ pcx_header_from_buffer (guint8 *buf)
 
   for (i = 0; pcx_header_buf_xlate[i].size != 0; i++)
     {
-      g_memmove (pcx_header_buf_xlate[i].address, buf + buf_offset,
-                 pcx_header_buf_xlate[i].size);
+      memmove (pcx_header_buf_xlate[i].address, buf + buf_offset,
+               pcx_header_buf_xlate[i].size);
       buf_offset += pcx_header_buf_xlate[i].size;
     }
 }
@@ -334,8 +345,8 @@ pcx_header_to_buffer (guint8 *buf)
 
   for (i = 0; pcx_header_buf_xlate[i].size != 0; i++)
     {
-      g_memmove (buf + buf_offset, pcx_header_buf_xlate[i].address,
-                 pcx_header_buf_xlate[i].size);
+      memmove (buf + buf_offset, pcx_header_buf_xlate[i].address,
+               pcx_header_buf_xlate[i].size);
       buf_offset += pcx_header_buf_xlate[i].size;
     }
 }
@@ -345,14 +356,17 @@ load_image (const gchar  *filename,
             GError      **error)
 {
   FILE         *fd;
-  GimpDrawable *drawable;
-  GimpPixelRgn  pixel_rgn;
+  GeglBuffer   *buffer;
   guint16       offset_x, offset_y, bytesperline;
   gint32        width, height;
+  guint16       resolution_x, resolution_y;
   gint32        image, layer;
   guchar       *dest, cmap[768];
   guint8        header_buf[128];
 
+  gimp_progress_init_printf (_("Opening '%s'"),
+                             gimp_filename_to_utf8 (filename));
+
   fd = g_fopen (filename, "rb");
 
   if (! fd)
@@ -363,9 +377,6 @@ load_image (const gchar  *filename,
       return -1;
     }
 
-  gimp_progress_init_printf (_("Opening '%s'"),
-                             gimp_filename_to_utf8 (filename));
-
   if (fread (header_buf, 128, 1, fd) == 0)
     {
       g_set_error (error, G_FILE_ERROR, G_FILE_ERROR_FAILED,
@@ -391,25 +402,34 @@ load_image (const gchar  *filename,
   width        = GUINT16_FROM_LE (pcx_header.x2) - offset_x + 1;
   height       = GUINT16_FROM_LE (pcx_header.y2) - offset_y + 1;
   bytesperline = GUINT16_FROM_LE (pcx_header.bytesperline);
+  resolution_x = GUINT16_FROM_LE (pcx_header.hdpi);
+  resolution_y = GUINT16_FROM_LE (pcx_header.vdpi);
 
-  if ((width < 0) || (width > GIMP_MAX_IMAGE_SIZE))
+  if ((width <= 0) || (width > GIMP_MAX_IMAGE_SIZE))
     {
       g_message (_("Unsupported or invalid image width: %d"), width);
       fclose (fd);
       return -1;
     }
-  if ((height < 0) || (height > GIMP_MAX_IMAGE_SIZE))
+  if ((height <= 0) || (height > GIMP_MAX_IMAGE_SIZE))
     {
       g_message (_("Unsupported or invalid image height: %d"), height);
       fclose (fd);
       return -1;
     }
-  if (bytesperline < (width * pcx_header.bpp) / 8)
+  if (bytesperline < ((width * pcx_header.bpp + 7) / 8))
     {
       g_message (_("Invalid number of bytes per line in PCX header"));
       fclose (fd);
       return -1;
     }
+  if ((resolution_x < 1) || (resolution_x > GIMP_MAX_RESOLUTION) ||
+      (resolution_y < 1) || (resolution_y > GIMP_MAX_RESOLUTION))
+    {
+      g_message (_("Resolution out of bounds in XCX header, using 72x72"));
+      resolution_x = 72;
+      resolution_y = 72;
+    }
 
   /* Shield against potential buffer overflows in load_*() functions. */
   if (G_MAXSIZE / width / height < 3)
@@ -423,32 +443,72 @@ load_image (const gchar  *filename,
     {
       image= gimp_image_new (width, height, GIMP_RGB);
       layer= gimp_layer_new (image, _("Background"), width, height,
-                             GIMP_RGB_IMAGE, 100, GIMP_NORMAL_MODE);
+                             GIMP_RGB_IMAGE,
+                             100, GIMP_LAYER_MODE_NORMAL_LEGACY);
     }
   else
     {
       image= gimp_image_new (width, height, GIMP_INDEXED);
       layer= gimp_layer_new (image, _("Background"), width, height,
-                             GIMP_INDEXED_IMAGE, 100, GIMP_NORMAL_MODE);
+                             GIMP_INDEXED_IMAGE,
+                             100, GIMP_LAYER_MODE_NORMAL_LEGACY);
     }
+
   gimp_image_set_filename (image, filename);
+  gimp_image_set_resolution (image, resolution_x, resolution_y);
+
   gimp_image_insert_layer (image, layer, -1, 0);
   gimp_layer_set_offsets (layer, offset_x, offset_y);
-  drawable = gimp_drawable_get (layer);
+
+  buffer = gimp_drawable_get_buffer (layer);
 
   if (pcx_header.planes == 1 && pcx_header.bpp == 1)
     {
       dest = g_new (guchar, ((gsize) width) * height);
       load_1 (fd, width, height, dest, bytesperline);
-      gimp_image_set_colormap (image, mono, 2);
+      /* Monochrome does not mean necessarily B&W. Therefore we still
+       * want to check the header palette, even for just 2 colors.
+       * Hopefully the header palette will always be filled with
+       * meaningful colors and the creator software did not just assume
+       * B&W by being monochrome.
+       * Until now test samples showed that even when B&W the header
+       * palette was correctly filled with these 2 colors and we didn't
+       * find counter-examples.
+       * See bug 159947, comment 21 and 23.
+       */
+      gimp_image_set_colormap (image, pcx_header.colormap, 2);
     }
-  else if (pcx_header.planes == 4 && pcx_header.bpp == 1)
+  else if (pcx_header.bpp == 1 && pcx_header.planes == 2)
+    {
+      dest = g_new (guchar, ((gsize) width) * height);
+      load_sub_8 (fd, width, height, 1, 2, dest, bytesperline);
+      gimp_image_set_colormap (image, pcx_header.colormap, 4);
+    }
+  else if (pcx_header.bpp == 2 && pcx_header.planes == 1)
+    {
+      dest = g_new (guchar, ((gsize) width) * height);
+      load_sub_8 (fd, width, height, 2, 1, dest, bytesperline);
+      gimp_image_set_colormap (image, pcx_header.colormap, 4);
+    }
+  else if (pcx_header.bpp == 1 && pcx_header.planes == 3)
+    {
+      dest = g_new (guchar, ((gsize) width) * height);
+      load_sub_8 (fd, width, height, 1, 3, dest, bytesperline);
+      gimp_image_set_colormap (image, pcx_header.colormap, 8);
+    }
+  else if (pcx_header.bpp == 1 && pcx_header.planes == 4)
     {
       dest = g_new (guchar, ((gsize) width) * height);
       load_4 (fd, width, height, dest, bytesperline);
       gimp_image_set_colormap (image, pcx_header.colormap, 16);
     }
-  else if (pcx_header.planes == 1 && pcx_header.bpp == 8)
+  else if (pcx_header.bpp == 4 && pcx_header.planes == 1)
+    {
+      dest = g_new (guchar, ((gsize) width) * height);
+      load_sub_8 (fd, width, height, 4, 1, dest, bytesperline);
+      gimp_image_set_colormap (image, pcx_header.colormap, 16);
+    }
+  else if (pcx_header.bpp == 8 && pcx_header.planes == 1)
     {
       dest = g_new (guchar, ((gsize) width) * height);
       load_8 (fd, width, height, dest, bytesperline);
@@ -456,7 +516,7 @@ load_image (const gchar  *filename,
       fread (cmap, 768, 1, fd);
       gimp_image_set_colormap (image, cmap, 256);
     }
-  else if (pcx_header.planes == 3 && pcx_header.bpp == 8)
+  else if (pcx_header.bpp == 8 && pcx_header.planes == 3)
     {
       dest = g_new (guchar, ((gsize) width) * height * 3);
       load_24 (fd, width, height, dest, bytesperline);
@@ -468,16 +528,14 @@ load_image (const gchar  *filename,
       return -1;
     }
 
-  gimp_progress_update (1.0);
-
-  gimp_pixel_rgn_init (&pixel_rgn, drawable, 0, 0, width, height, TRUE, FALSE);
-  gimp_pixel_rgn_set_rect (&pixel_rgn, dest, 0, 0, width, height);
+  gegl_buffer_set (buffer, GEGL_RECTANGLE (0, 0, width, height), 0,
+                   NULL, dest, GEGL_AUTO_ROWSTRIDE);
 
   fclose (fd);
   g_free (dest);
+  g_object_unref (buffer);
 
-  gimp_drawable_flush (drawable);
-  gimp_drawable_detach (drawable);
+  gimp_progress_update (1.0);
 
   return image;
 }
@@ -486,16 +544,16 @@ static void
 load_8 (FILE    *fp,
         gint     width,
         gint     height,
-        guchar  *buffer,
+        guchar  *buf,
         guint16  bytes)
 {
   gint    row;
   guchar *line = g_new (guchar, bytes);
 
-  for (row = 0; row < height; buffer += width, ++row)
+  for (row = 0; row < height; buf += width, ++row)
     {
       readline (fp, line, bytes);
-      memcpy (buffer, line, width);
+      memcpy (buf, line, width);
       gimp_progress_update ((double) row / (double) height);
     }
 
@@ -506,20 +564,20 @@ static void
 load_24 (FILE    *fp,
          gint     width,
          gint     height,
-         guchar  *buffer,
+         guchar  *buf,
          guint16  bytes)
 {
   gint    x, y, c;
   guchar *line = g_new (guchar, bytes);
 
-  for (y = 0; y < height; buffer += width * 3, ++y)
+  for (y = 0; y < height; buf += width * 3, ++y)
     {
       for (c = 0; c < 3; ++c)
         {
           readline (fp, line, bytes);
           for (x = 0; x < width; ++x)
             {
-              buffer[x * 3 + c] = line[x];
+              buf[x * 3 + c] = line[x];
             }
         }
       gimp_progress_update ((double) y / (double) height);
@@ -532,21 +590,21 @@ static void
 load_1 (FILE    *fp,
         gint     width,
         gint     height,
-        guchar  *buffer,
+        guchar  *buf,
         guint16  bytes)
 {
   gint    x, y;
   guchar *line = g_new (guchar, bytes);
 
-  for (y = 0; y < height; buffer += width, ++y)
+  for (y = 0; y < height; buf += width, ++y)
     {
       readline (fp, line, bytes);
       for (x = 0; x < width; ++x)
         {
           if (line[x / 8] & (128 >> (x % 8)))
-            buffer[x] = 1;
+            buf[x] = 1;
           else
-            buffer[x] = 0;
+            buf[x] = 0;
         }
       gimp_progress_update ((double) y / (double) height);
     }
@@ -558,23 +616,60 @@ static void
 load_4 (FILE    *fp,
         gint     width,
         gint     height,
-        guchar  *buffer,
+        guchar  *buf,
         guint16  bytes)
 {
   gint    x, y, c;
   guchar *line = g_new (guchar, bytes);
 
-  for (y = 0; y < height; buffer += width, ++y)
+  for (y = 0; y < height; buf += width, ++y)
     {
       for (x = 0; x < width; ++x)
-        buffer[x] = 0;
+        buf[x] = 0;
       for (c = 0; c < 4; ++c)
         {
           readline(fp, line, bytes);
           for (x = 0; x < width; ++x)
             {
               if (line[x / 8] & (128 >> (x % 8)))
-                buffer[x] += (1 << c);
+                buf[x] += (1 << c);
+            }
+        }
+      gimp_progress_update ((double) y / (double) height);
+    }
+
+  g_free (line);
+}
+
+static void
+load_sub_8 (FILE    *fp,
+            gint     width,
+            gint     height,
+            gint     bpp,
+            gint     plane,
+            guchar  *buf,
+            guint16  bytes)
+{
+  gint    x, y, c, b;
+  guchar *line = g_new (guchar, bytes);
+  gint    real_bpp = bpp - 1;
+  gint    current_bit = 0;
+
+  for (y = 0; y < height; buf += width, ++y)
+    {
+      for (x = 0; x < width; ++x)
+        buf[x] = 0;
+      for (c = 0; c < plane; ++c)
+        {
+          readline (fp, line, bytes);
+          for (x = 0; x < width; ++x)
+            {
+              for (b = 0; b < bpp; b++)
+                {
+                  current_bit = bpp * x + b;
+                  if (line[current_bit / 8] & (128 >> (current_bit % 8)))
+                    buf[x] += (1 << (real_bpp - b + c));
+                }
             }
         }
       gimp_progress_update ((double) y / (double) height);
@@ -585,7 +680,7 @@ load_4 (FILE    *fp,
 
 static void
 readline (FILE   *fp,
-          guchar *buffer,
+          guchar *buf,
           gint    bytes)
 {
   static guchar count = 0, value = 0;
@@ -608,12 +703,12 @@ readline (FILE   *fp,
                 }
             }
           count--;
-          *(buffer++) = value;
+          *(buf++) = value;
         }
     }
   else
     {
-      fread (buffer, bytes, 1, fp);
+      fread (buf, bytes, 1, fp);
     }
 }
 
@@ -624,24 +719,26 @@ save_image (const gchar  *filename,
             GError      **error)
 {
   FILE          *fp;
-  GimpPixelRgn   pixel_rgn;
-  GimpDrawable  *drawable;
+  GeglBuffer    *buffer;
+  const Babl    *format;
   GimpImageType  drawable_type;
   guchar        *cmap= NULL;
   guchar        *pixels;
   gint           offset_x, offset_y;
   guint          width, height;
+  gdouble        resolution_x, resolution_y;
   gint           colors, i;
   guint8         header_buf[128];
 
-  drawable = gimp_drawable_get (layer);
   drawable_type = gimp_drawable_type (layer);
   gimp_drawable_offsets (layer, &offset_x, &offset_y);
-  width = drawable->width;
-  height = drawable->height;
-  gimp_pixel_rgn_init (&pixel_rgn, drawable, 0, 0, width, height, FALSE, FALSE);
 
-  gimp_progress_init_printf (_("Saving '%s'"),
+  buffer = gimp_drawable_get_buffer (layer);
+
+  width  = gegl_buffer_get_width  (buffer);
+  height = gegl_buffer_get_height (buffer);
+
+  gimp_progress_init_printf (_("Exporting '%s'"),
                              gimp_filename_to_utf8 (filename));
 
   pcx_header.manufacturer = 0x0a;
@@ -651,34 +748,77 @@ save_image (const gchar  *filename,
   switch (drawable_type)
     {
     case GIMP_INDEXED_IMAGE:
-      cmap = gimp_image_get_colormap (image, &colors);
-      pcx_header.bpp = 8;
-      pcx_header.bytesperline = GUINT16_TO_LE (width);
-      pcx_header.planes = 1;
-      pcx_header.color = GUINT16_TO_LE (1);
+      cmap                          = gimp_image_get_colormap (image, &colors);
+      if (colors > 16)
+        {
+          pcx_header.bpp            = 8;
+          pcx_header.planes         = 1;
+          pcx_header.bytesperline   = GUINT16_TO_LE (width);
+        }
+      else if (colors > 2)
+        {
+          pcx_header.bpp            = 4;
+          pcx_header.planes         = 1;
+          pcx_header.bytesperline   = GUINT16_TO_LE ((width + 1) / 2);
+        }
+      else
+       {
+          pcx_header.bpp            = 1;
+          pcx_header.planes         = 1;
+          pcx_header.bytesperline   = GUINT16_TO_LE ((width + 7) / 8);
+        }
+      pcx_header.color        = GUINT16_TO_LE (1);
+      format                  = NULL;
+
+      /* Some references explain that 2bpp/1plane and 4bpp/1plane files
+       * would use the palette at EOF (not the one from the header) if
+       * we are in version 5 of PCX. Other sources affirm that even in
+       * version 5, EOF palette must be used only when there are more
+       * than 16 colors. We go with this second assumption.
+       * See bug 159947, comment 21 and 23.
+       */
+      if (colors <= 16)
+        {
+          for (i = 0; i < (colors * 3); i++)
+            {
+              pcx_header.colormap[i] = cmap[i];
+            }
+        }
+
       break;
 
     case GIMP_RGB_IMAGE:
-      pcx_header.bpp = 8;
-      pcx_header.planes = 3;
-      pcx_header.color = GUINT16_TO_LE (1);
+      pcx_header.bpp          = 8;
+      pcx_header.planes       = 3;
+      pcx_header.color        = GUINT16_TO_LE (1);
       pcx_header.bytesperline = GUINT16_TO_LE (width);
+      format                  = babl_format ("R'G'B' u8");
       break;
 
     case GIMP_GRAY_IMAGE:
-      pcx_header.bpp = 8;
-      pcx_header.planes = 1;
-      pcx_header.color = GUINT16_TO_LE (2);
+      pcx_header.bpp          = 8;
+      pcx_header.planes       = 1;
+      pcx_header.color        = GUINT16_TO_LE (2);
       pcx_header.bytesperline = GUINT16_TO_LE (width);
+      format                  = babl_format ("Y' u8");
       break;
 
     default:
-      g_message (_("Cannot save images with alpha channel."));
+      g_message (_("Cannot export images with alpha channel."));
       return FALSE;
   }
 
+  /* Bytes per Line must be an even number, according to spec */
+  if (pcx_header.bytesperline % 2 != 0)
+    {
+      pcx_header.bytesperline++;
+    }
+
   pixels = (guchar *) g_malloc (width * height * pcx_header.planes);
-  gimp_pixel_rgn_get_rect (&pixel_rgn, pixels, 0, 0, width, height);
+
+  gegl_buffer_get (buffer, GEGL_RECTANGLE (0, 0, width, height), 1.0,
+                   format, pixels,
+                   GEGL_AUTO_ROWSTRIDE, GEGL_ABYSS_NONE);
 
   if ((offset_x < 0) || (offset_x > (1<<16)))
     {
@@ -719,8 +859,10 @@ save_image (const gchar  *filename,
   pcx_header.x2 = GUINT16_TO_LE ((guint16)(offset_x + width - 1));
   pcx_header.y2 = GUINT16_TO_LE ((guint16)(offset_y + height - 1));
 
-  pcx_header.hdpi = GUINT16_TO_LE (300);
-  pcx_header.vdpi = GUINT16_TO_LE (300);
+  gimp_image_get_resolution (image, &resolution_x, &resolution_y);
+
+  pcx_header.hdpi = GUINT16_TO_LE (RINT (MAX (resolution_x, 1.0)));
+  pcx_header.vdpi = GUINT16_TO_LE (RINT (MAX (resolution_y, 1.0)));
   pcx_header.reserved = 0;
 
   pcx_header_to_buffer (header_buf);
@@ -730,14 +872,21 @@ save_image (const gchar  *filename,
   switch (drawable_type)
     {
     case GIMP_INDEXED_IMAGE:
-      save_8 (fp, width, height, pixels);
-      fputc (0x0c, fp);
-      fwrite (cmap, colors, 3, fp);
-      for (i = colors; i < 256; i++)
+      if (colors > 16)
         {
-          fputc (0, fp);
-          fputc (0, fp);
-          fputc (0, fp);
+          save_8 (fp, width, height, pixels);
+          fputc (0x0c, fp);
+          fwrite (cmap, colors, 3, fp);
+          for (i = colors; i < 256; i++)
+            {
+              fputc (0, fp);
+              fputc (0, fp);
+              fputc (0, fp);
+            }
+        }
+      else /* Covers 1 and 4 bpp */
+        {
+          save_less_than_8 (fp, width, height, pcx_header.bpp, pixels);
         }
       break;
 
@@ -760,7 +909,7 @@ save_image (const gchar  *filename,
       return FALSE;
     }
 
-  gimp_drawable_detach (drawable);
+  g_object_unref (buffer);
   g_free (pixels);
 
   if (fclose (fp) != 0)
@@ -774,18 +923,58 @@ save_image (const gchar  *filename,
   return TRUE;
 }
 
+static void
+save_less_than_8 (FILE         *fp,
+                  gint          width,
+                  gint          height,
+                  const gint    bpp,
+                  const guchar *buf)
+{
+  const gint  bit_limit     = (8 - bpp);
+  const gint  buf_size      = width * height;
+  const gint  line_end      = width - 1;
+  gint        j             = bit_limit;
+  gint        count         = 0;
+  guchar      byte_to_write = 0x00;
+  guchar     *line;
+
+  line = (guchar *) g_malloc (((width + 7) / 8) * bpp);
+
+  for (gint x = 0; x < buf_size; x++)
+    {
+      byte_to_write |= (buf[x] << j);
+      j -= bpp;
+
+      if (j < 0 || (x % width == line_end))
+        {
+          line[count] = byte_to_write;
+          count++;
+          byte_to_write = 0x00;
+          j = bit_limit;
+
+          if ((x % width == line_end))
+            {
+              writeline (fp, line, count);
+              count = 0;
+              gimp_progress_update ((double) x / (double) buf_size);
+            }
+        }
+    }
+  g_free (line);
+}
+
 static void
 save_8 (FILE         *fp,
         gint          width,
         gint          height,
-        const guchar *buffer)
+        const guchar *buf)
 {
   int row;
 
   for (row = 0; row < height; ++row)
     {
-      writeline (fp, buffer, width);
-      buffer += width;
+      writeline (fp, buf, width);
+      buf += width;
       gimp_progress_update ((double) row / (double) height);
     }
 }
@@ -794,7 +983,7 @@ static void
 save_24 (FILE         *fp,
          gint          width,
          gint          height,
-         const guchar *buffer)
+         const guchar *buf)
 {
   int     x, y, c;
   guchar *line;
@@ -807,11 +996,11 @@ save_24 (FILE         *fp,
         {
           for (x = 0; x < width; ++x)
             {
-              line[x] = buffer[(3*x) + c];
+              line[x] = buf[(3*x) + c];
             }
           writeline (fp, line, width);
         }
-      buffer += width * 3;
+      buf += width * 3;
       gimp_progress_update ((double) y / (double) height);
     }
   g_free (line);
@@ -819,21 +1008,21 @@ save_24 (FILE         *fp,
 
 static void
 writeline (FILE         *fp,
-           const guchar *buffer,
+           const guchar *buf,
            gint          bytes)
 {
-  const guchar *finish = buffer + bytes;
+  const guchar *finish = buf + bytes;
   guchar        value;
   guchar        count;
 
-  while (buffer < finish)
+  while (buf < finish)
     {
-      value = *(buffer++);
+      value = *(buf++);
       count = 1;
 
-      while (buffer < finish && count < 63 && *buffer == value)
+      while (buf < finish && count < 63 && *buf == value)
         {
-          count++; buffer++;
+          count++; buf++;
         }
 
       if (value < 0xc0 && count == 1)
