@@ -2,9 +2,9 @@
  * Copyright (C) 1995 Spencer Kimball and Peter Mattis
  * Copyright (C) 1997 Daniel Risacher
  *
- * This program is free software; you can redistribute it and/or modify
+ * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -13,8 +13,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 /*
@@ -33,8 +32,10 @@
 
 #include <string.h>
 
+#ifdef SENDMAIL
 #include <sys/types.h>
 #include <sys/wait.h>
+#endif
 
 #include <glib/gstdio.h>
 
@@ -43,74 +44,11 @@
 
 #include "libgimp/stdplugins-intl.h"
 
-
-/* GdkPixbuf RGBA C-Source image dump 1-byte-run-length-encoded */
-
-#ifdef __SUNPRO_C
-#pragma align 4 (mail_icon)
-#endif
-#ifdef __GNUC__
-static const guint8 mail_icon[] __attribute__ ((__aligned__ (4))) =
-#else
-static const guint8 mail_icon[] =
-#endif
-{ ""
-  /* Pixbuf magic (0x47646b50) */
-  "GdkP"
-  /* length: header (24) + pixel_data (584) */
-  "\0\0\2`"
-  /* pixdata_type (0x2010002) */
-  "\2\1\0\2"
-  /* rowstride (64) */
-  "\0\0\0@"
-  /* width (16) */
-  "\0\0\0\20"
-  /* height (16) */
-  "\0\0\0\20"
-  /* pixel_data: */
-  "\245\0\0\0\0\1\210\212\2050\205\210\212\205\377\1\210\212\2050\210\0"
-  "\0\0\0\2\210\212\2050\210\212\205\377\205\377\377\377\377\2\210\212\205"
-  "\377\210\212\2050\206\0\0\0\0\13\210\212\2050\210\212\205\377\377\377"
-  "\377\377\331\326\320\377\311\307\301\377\301\277\272\377\257\254\251"
-  "\377\241\240\232\377\377\377\377\377\210\212\205\377\210\212\2050\204"
-  "\0\0\0\0\15\210\212\2050\210\212\205\377\377\377\377\377\331\326\320"
-  "\377\250\247\241\377\276\275\271\377\316\315\312\377\317\316\314\377"
-  "\306\305\305\377\275\275\275\377\377\377\377\377\210\212\205\377\210"
-  "\212\2050\203\0\0\0\0\7\210\212\205\377\344\344\343\377\271\270\263\377"
-  "\225\224\216\377\333\332\331\377\354\354\353\377\365\365\365\377\202"
-  "\377\377\377\377\4\305\305\305\377\261\256\252\377\352\352\351\377\210"
-  "\212\205\377\203\0\0\0\0\4\210\212\205\377\367\367\367\377\257\256\254"
-  "\377\365\365\365\377\204\377\377\377\377\5\353\353\353\377\330\330\330"
-  "\377\315\314\311\377\367\367\367\377\210\212\205\377\203\0\0\0\0\4\210"
-  "\212\205\377\377\377\377\377\346\345\342\377\252\251\247\377\202\353"
-  "\353\353\377\7\341\341\341\377\330\330\330\377\317\317\317\377\246\245"
-  "\241\377\363\363\362\377\377\377\377\377\210\212\205\377\203\0\0\0\0"
-  "\5\210\212\205\377\377\377\377\377\373\373\370\377\342\342\337\377\222"
-  "\221\221\377\202\266\264\263\377\6\274\273\267\377\241\240\240\377\363"
-  "\363\362\377\367\365\363\377\377\377\377\377\210\212\205\377\203\0\0"
-  "\0\0\5\210\212\205\377\377\377\377\377\373\373\370\377\360\357\355\377"
-  "\245\242\235\377\203\373\373\370\377\5\220\215\207\377\371\367\363\377"
-  "\362\360\354\377\377\377\377\377\210\212\205\377\203\0\0\0\0\4\210\212"
-  "\205\377\377\377\377\377\373\372\367\377\262\261\256\377\203\373\373"
-  "\370\377\202\371\370\364\377\4\274\273\267\377\351\347\342\377\377\377"
-  "\377\377\210\212\205\377\203\0\0\0\0\15\210\212\205\377\377\377\377\377"
-  "\311\307\304\377\373\373\370\377\367\365\361\377\372\371\366\377\370"
-  "\366\362\377\372\370\365\377\367\365\362\377\353\350\343\377\317\316"
-  "\312\377\377\377\377\377\210\212\205\377\203\0\0\0\0\2\210\212\205\377"
-  "\360\360\360\377\211\377\377\377\377\2\360\360\360\377\210\212\205\377"
-  "\203\0\0\0\0\1\210\212\205\217\213\210\212\205\377\1\210\212\205\217"
-  "\221\0\0\0\0"
-};
-
-
-#ifndef SENDMAIL
-#define SENDMAIL "/usr/lib/sendmail"
-#endif
-
 #define BUFFER_SIZE 256
 
 #define PLUG_IN_PROC   "plug-in-mail-image"
 #define PLUG_IN_BINARY "mail"
+#define PLUG_IN_ROLE   "gimp-mail"
 
 typedef struct
 {
@@ -122,32 +60,36 @@ typedef struct
 } m_info;
 
 
-static void               query                (void);
-static void               run                  (const gchar      *name,
-                                                gint              nparams,
-                                                const GimpParam  *param,
-                                                gint             *nreturn_vals,
-                                                GimpParam       **return_vals);
+static void               query                   (void);
+static void               run                     (const gchar      *name,
+                                                   gint              nparams,
+                                                   const GimpParam  *param,
+                                                   gint             *nreturn_vals,
+                                                   GimpParam       **return_vals);
 
-static GimpPDBStatusType  save_image           (const gchar      *filename,
-                                                gint32            image_ID,
-                                                gint32            drawable_ID,
-                                                gint32            run_mode);
+static GimpPDBStatusType  send_image              (const gchar      *filename,
+                                                   gint32            image_ID,
+                                                   gint32            drawable_ID,
+                                                   gint32            run_mode);
 
-static gboolean           save_dialog          (void);
-static void               mail_entry_callback  (GtkWidget        *widget,
-                                                gchar            *data);
-static void               mesg_body_callback   (GtkTextBuffer    *buffer,
-                                                gpointer          data);
+static gboolean           send_dialog             (void);
+static void               mail_entry_callback     (GtkWidget        *widget,
+                                                   gchar            *data);
+static gboolean           valid_file              (const gchar      *filename);
+static gchar            * find_extension          (const gchar      *filename);
 
-static gboolean           valid_file           (const gchar      *filename);
-static void               create_headers       (FILE             *mailpipe);
-static gchar            * find_extension       (const gchar      *filename);
-static gboolean           to64                 (const gchar      *filename,
-                                                FILE             *outfile,
-                                                GError          **error);
-static FILE             * sendmail_pipe        (gchar           **cmd,
-                                                GPid             *pid);
+#ifdef SENDMAIL
+static void               mesg_body_callback      (GtkTextBuffer    *buffer,
+                                                   gpointer          data);
+
+static gchar            * sendmail_content_type   (const gchar      *filename);
+static void               sendmail_create_headers (FILE             *mailpipe);
+static gboolean           sendmail_to64           (const gchar      *filename,
+                                                   FILE             *outfile,
+                                                   GError          **error);
+static FILE             * sendmail_pipe           (gchar           **cmd,
+                                                   GPid             *pid);
+#endif
 
 
 const GimpPlugInInfo PLUG_IN_INFO =
@@ -171,9 +113,11 @@ MAIN ()
 static void
 query (void)
 {
+  gchar *email_bin;
+
   static const GimpParamDef args[] =
   {
-    { GIMP_PDB_INT32,    "run-mode",      "Interactive, non-interactive" },
+    { GIMP_PDB_INT32,    "run-mode",      "The run mode { RUN-INTERACTIVE (0), RUN-NONINTERACTIVE (1) }" },
     { GIMP_PDB_IMAGE,    "image",         "Input image" },
     { GIMP_PDB_DRAWABLE, "drawable",      "Drawable to save" },
     { GIMP_PDB_STRING,   "filename",      "The name of the file to save the image in" },
@@ -184,22 +128,54 @@ query (void)
     { GIMP_PDB_INT32,    "encapsulation", "ignored" }
   };
 
+  /* Check if xdg-email or sendmail is installed.
+   * TODO: allow setting the location of the executable in preferences.
+   */
+#ifdef SENDMAIL
+  if (strlen (SENDMAIL) == 0)
+    {
+      email_bin = g_find_program_in_path ("sendmail");
+    }
+  else
+    {
+      /* If a directory has been set at build time, we assume that sendmail
+       * can only be in this directory. */
+      email_bin = g_build_filename (SENDMAIL, "sendmail", NULL);
+      if (! g_file_test (email_bin, G_FILE_TEST_IS_EXECUTABLE))
+        {
+          g_free (email_bin);
+          email_bin = NULL;
+        }
+    }
+#else
+  email_bin = g_find_program_in_path ("xdg-email");
+#endif
+
+  if (email_bin == NULL)
+    return;
+
   gimp_install_procedure (PLUG_IN_PROC,
-			  N_("Send the image by email"),
-			  "You need to have sendmail installed",
-			  "Adrian Likins, Reagan Blundell",
-			  "Adrian Likins, Reagan Blundell, Daniel Risacher, "
+                          N_("Send the image by email"),
+#ifdef SENDMAIL
+                          "Sendmail is used to send emails and must be properly configured.",
+#else /* xdg-email */
+                          "The preferred email composer is used to send emails and must be properly configured.",
+#endif
+                          "Adrian Likins, Reagan Blundell",
+                          "Adrian Likins, Reagan Blundell, Daniel Risacher, "
                           "Spencer Kimball and Peter Mattis",
-			  "1995-1997",
-			  N_("Send by E_mail..."),
-			  "*",
-			  GIMP_PLUGIN,
-			  G_N_ELEMENTS (args), 0,
-			  args, NULL);
+                          "1995-1997",
+                          N_("Send by E_mail..."),
+                          "*",
+                          GIMP_PLUGIN,
+                          G_N_ELEMENTS (args), 0,
+                          args, NULL);
 
   gimp_plugin_menu_register (PLUG_IN_PROC, "<Image>/File/Send");
-  gimp_plugin_icon_register (PLUG_IN_PROC, GIMP_ICON_TYPE_INLINE_PIXBUF,
-                             mail_icon);
+  gimp_plugin_icon_register (PLUG_IN_PROC, GIMP_ICON_TYPE_ICON_NAME,
+                             (const guint8 *) GIMP_ICON_EDIT);
+
+  g_free (email_bin);
 }
 
 static void
@@ -246,47 +222,47 @@ run (const gchar      *name,
               }
           }
 
-	  if (! save_dialog ())
-	    status = GIMP_PDB_CANCEL;
-	  break;
+          if (! send_dialog ())
+            status = GIMP_PDB_CANCEL;
+          break;
 
-	case GIMP_RUN_NONINTERACTIVE:
-	  /*  Make sure all the arguments are there!  */
-	  if (nparams < 8)
-	    {
-	      status = GIMP_PDB_CALLING_ERROR;
-	    }
-	  else
-	    {
-	      g_strlcpy (mail_info.filename,
+        case GIMP_RUN_NONINTERACTIVE:
+          /*  Make sure all the arguments are there!  */
+          if (nparams < 8)
+            {
+              status = GIMP_PDB_CALLING_ERROR;
+            }
+          else
+            {
+              g_strlcpy (mail_info.filename,
                          param[3].data.d_string, BUFFER_SIZE);
-	      g_strlcpy (mail_info.receipt,
+              g_strlcpy (mail_info.receipt,
                          param[4].data.d_string, BUFFER_SIZE);
-	      g_strlcpy (mail_info.from,
+              g_strlcpy (mail_info.from,
                          param[5].data.d_string, BUFFER_SIZE);
-	      g_strlcpy (mail_info.subject,
+              g_strlcpy (mail_info.subject,
                          param[6].data.d_string, BUFFER_SIZE);
-	      g_strlcpy (mail_info.comment,
+              g_strlcpy (mail_info.comment,
                          param[7].data.d_string, BUFFER_SIZE);
-	    }
-	  break;
+            }
+          break;
 
-	case GIMP_RUN_WITH_LAST_VALS:
-	  gimp_get_data (PLUG_IN_PROC, &mail_info);
-	  break;
+        case GIMP_RUN_WITH_LAST_VALS:
+          gimp_get_data (PLUG_IN_PROC, &mail_info);
+          break;
 
-	default:
-	  break;
-	}
+        default:
+          break;
+        }
 
       if (status == GIMP_PDB_SUCCESS)
-	{
-	  status = save_image (mail_info.filename,
-			       image_ID,
-			       drawable_ID,
-			       run_mode);
+        {
+          status = send_image (mail_info.filename,
+                               image_ID,
+                               drawable_ID,
+                               run_mode);
 
-	  if (status == GIMP_PDB_SUCCESS)
+          if (status == GIMP_PDB_SUCCESS)
             {
               if (mesg_body)
                 g_strlcpy (mail_info.comment, mesg_body, BUFFER_SIZE);
@@ -304,17 +280,25 @@ run (const gchar      *name,
 }
 
 static GimpPDBStatusType
-save_image (const gchar *filename,
-	    gint32       image_ID,
-	    gint32       drawable_ID,
-	    gint32       run_mode)
+send_image (const gchar *filename,
+            gint32       image_ID,
+            gint32       drawable_ID,
+            gint32       run_mode)
 {
   GimpPDBStatusType  status = GIMP_PDB_SUCCESS;
   gchar             *ext;
   gchar             *tmpname;
+#ifndef SENDMAIL /* xdg-email */
+  gchar             *mailcmd[9];
+  gchar             *filepath = NULL;
+  GFile             *tmp_dir  = NULL;
+  GFileEnumerator   *enumerator;
+  gint               i;
+#else /* SENDMAIL */
   gchar             *mailcmd[3];
   GPid               mailpid;
-  FILE              *mailpipe;
+  FILE              *mailpipe = NULL;
+#endif
   GError            *error = NULL;
 
   ext = find_extension (filename);
@@ -325,8 +309,107 @@ save_image (const gchar *filename,
   /* get a temp name with the right extension and save into it. */
   tmpname = gimp_temp_name (ext + 1);
 
+  if (! (gimp_file_save (run_mode,
+                         image_ID,
+                         drawable_ID,
+                         tmpname,
+                         tmpname) && valid_file (tmpname)))
+    {
+      goto error;
+    }
+
+#ifndef SENDMAIL /* xdg-email */
+  /* From xdg-email doc:
+   * "Some e-mail applications require the file to remain present
+   * after xdg-email returns."
+   * As a consequence, the file cannot be removed at the end of the
+   * function. We actually have no way to ever know *when* the file can
+   * be removed since the caller could leave the email window opened for
+   * hours. Yet we still want to clean sometimes and not have temporary
+   * images piling up.
+   * So I use a known directory that we control under $GIMP_DIRECTORY/tmp/,
+   * and clean it out each time the plugin runs. This means that *if* you
+   * are in the above case (your email client requires the file to stay
+   * alive), * you cannot run twice the plugin at the same time.
+   */
+  tmp_dir = gimp_directory_file ("tmp", PLUG_IN_PROC, NULL);
+
+  if (g_mkdir_with_parents (gimp_file_get_utf8_name (tmp_dir),
+                            S_IRUSR | S_IWUSR | S_IXUSR) == -1)
+    {
+      g_message ("Temporary directory %s could not be created.",
+                 gimp_file_get_utf8_name (tmp_dir));
+      g_error_free (error);
+      goto error;
+    }
+
+  enumerator = g_file_enumerate_children (tmp_dir,
+                                          G_FILE_ATTRIBUTE_STANDARD_TYPE,
+                                          G_FILE_QUERY_INFO_NOFOLLOW_SYMLINKS,
+                                          NULL, NULL);
+  if (enumerator)
+    {
+      GFileInfo *info;
+
+      while ((info = g_file_enumerator_next_file (enumerator,
+                                                  NULL, NULL)))
+        {
+          if (g_file_info_get_file_type (info) == G_FILE_TYPE_REGULAR)
+            {
+              GFile *file = g_file_enumerator_get_child (enumerator, info);
+              g_file_delete (file, NULL, NULL);
+              g_object_unref (file);
+            }
+
+          g_object_unref (info);
+        }
+
+      g_object_unref (enumerator);
+    }
+
+  filepath = g_build_filename (gimp_file_get_utf8_name (tmp_dir),
+                               mail_info.filename, NULL);
+  g_rename (tmpname, filepath);
+
+  mailcmd[0] = g_strdup ("xdg-email");
+  mailcmd[1] = "--attach";
+  mailcmd[2] = filepath;
+  i = 3;
+  if (mail_info.subject &&
+      strlen (mail_info.subject) > 0)
+    {
+      mailcmd[i++] = "--subject";
+      mailcmd[i++] = mail_info.subject;
+    }
+  if (mail_info.comment &&
+      strlen (mail_info.comment) > 0)
+    {
+      mailcmd[i++] = "--body";
+      mailcmd[i++] = mail_info.comment;
+    }
+  if (mail_info.receipt &&
+      strlen (mail_info.receipt) > 0)
+    {
+      mailcmd[i++] = mail_info.receipt;
+    }
+  mailcmd[i] = NULL;
+
+  if (! g_spawn_async (NULL, mailcmd, NULL,
+                       G_SPAWN_SEARCH_PATH,
+                       NULL, NULL, NULL, &error))
+    {
+      g_message ("%s", error->message);
+      g_error_free (error);
+      goto error;
+    }
+
+#else /* SENDMAIL */
   /* construct the "sendmail user@location" line */
-  mailcmd[0] = SENDMAIL;
+  if (strlen (SENDMAIL) == 0)
+    mailcmd[0] = g_strdup ("sendmail");
+  else
+    mailcmd[0] = g_build_filename (SENDMAIL, "sendmail", NULL);
+
   mailcmd[1] = mail_info.receipt;
   mailcmd[2] = NULL;
 
@@ -336,43 +419,49 @@ save_image (const gchar *filename,
   if (mailpipe == NULL)
     return GIMP_PDB_EXECUTION_ERROR;
 
-  create_headers (mailpipe);
+  sendmail_create_headers (mailpipe);
 
   fflush (mailpipe);
 
-  if (! (gimp_file_save (run_mode,
-			 image_ID,
-			 drawable_ID,
-			 tmpname,
-			 tmpname) && valid_file (tmpname)))
+  if (! sendmail_to64 (tmpname, mailpipe, &error))
     {
-      goto error;
-    }
-
-  if (! to64 (tmpname, mailpipe, &error))
-    {
-      g_message (error->message);
+      g_message ("%s", error->message);
       g_error_free (error);
       goto error;
     }
 
   fprintf (mailpipe, "\n--GUMP-MIME-boundary--\n");
+#endif
 
   goto cleanup;
 
 error:
   /* stop sendmail from doing anything */
+#ifdef SENDMAIL
   kill (mailpid, SIGINT);
+#endif
   status = GIMP_PDB_EXECUTION_ERROR;
 
 cleanup:
   /* close out the sendmail process */
-  fclose (mailpipe);
-  waitpid (mailpid, NULL, 0);
-  g_spawn_close_pid (mailpid);
+#ifdef SENDMAIL
+  if (mailpipe)
+    {
+      fclose (mailpipe);
+      waitpid (mailpid, NULL, 0);
+      g_spawn_close_pid (mailpid);
+    }
 
   /* delete the tmpfile that was generated */
   g_unlink (tmpname);
+#else
+  if (tmp_dir)
+    g_object_unref (tmp_dir);
+  if (filepath)
+    g_free (filepath);
+#endif
+
+  g_free (mailcmd[0]);
   g_free (tmpname);
 
   return status;
@@ -380,15 +469,17 @@ cleanup:
 
 
 static gboolean
-save_dialog (void)
+send_dialog (void)
 {
   GtkWidget     *dlg;
   GtkWidget     *main_vbox;
   GtkWidget     *entry;
   GtkWidget     *table;
+#ifdef SENDMAIL
   GtkWidget     *scrolled_window;
   GtkWidget     *text_view;
   GtkTextBuffer *text_buffer;
+#endif
   gchar         *gump_from;
   gint           row = 0;
   gboolean       run;
@@ -404,14 +495,14 @@ save_dialog (void)
       g_free (gump_from);
     }
 
-  dlg = gimp_dialog_new (_("Send by Email"), PLUG_IN_BINARY,
+  dlg = gimp_dialog_new (_("Send by Email"), PLUG_IN_ROLE,
                          NULL, 0,
-			 gimp_standard_help_func, PLUG_IN_PROC,
+                         gimp_standard_help_func, PLUG_IN_PROC,
 
-			 GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-			 _("_Send"),       GTK_RESPONSE_OK,
+                         _("_Cancel"), GTK_RESPONSE_CANCEL,
+                         _("_Send"),   GTK_RESPONSE_OK,
 
-			 NULL);
+                         NULL);
 
   gtk_dialog_set_alternative_button_order (GTK_DIALOG (dlg),
                                            GTK_RESPONSE_OK,
@@ -420,9 +511,9 @@ save_dialog (void)
 
   gimp_window_set_transient (GTK_WINDOW (dlg));
 
-  main_vbox = gtk_vbox_new (FALSE, 12);
+  main_vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 12);
   gtk_container_set_border_width (GTK_CONTAINER (main_vbox), 12);
-  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dlg)->vbox),
+  gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (dlg))),
                       main_vbox, TRUE, TRUE, 0);
   gtk_widget_show (main_vbox);
 
@@ -441,20 +532,22 @@ save_dialog (void)
   gtk_entry_set_max_length (GTK_ENTRY (entry), BUFFER_SIZE - 1);
   gtk_entry_set_text (GTK_ENTRY (entry), mail_info.filename);
   gimp_table_attach_aligned (GTK_TABLE (table), 0, row++,
-			     _("_Filename:"), 0.0, 0.5,
-			     entry, 1, FALSE);
+                             _("_Filename:"), 0.0, 0.5,
+                             entry, 1, FALSE);
   g_signal_connect (entry, "changed",
                     G_CALLBACK (mail_entry_callback),
                     mail_info.filename);
+  gtk_entry_set_activates_default (GTK_ENTRY (entry), TRUE);
 
+#ifdef SENDMAIL
   /* To entry */
   entry = gtk_entry_new ();
   gtk_widget_set_size_request (entry, 200, -1);
   gtk_entry_set_max_length (GTK_ENTRY (entry), BUFFER_SIZE - 1);
   gtk_entry_set_text (GTK_ENTRY (entry), mail_info.receipt);
   gimp_table_attach_aligned (GTK_TABLE (table), 0, row++,
-			     _("_To:"), 0.0, 0.5,
-			     entry, 1, FALSE);
+                             C_("email-address", "_To:"), 0.0, 0.5,
+                             entry, 1, FALSE);
   g_signal_connect (entry, "changed",
                     G_CALLBACK (mail_entry_callback),
                     mail_info.receipt);
@@ -467,8 +560,8 @@ save_dialog (void)
   gtk_entry_set_max_length (GTK_ENTRY (entry), BUFFER_SIZE - 1);
   gtk_entry_set_text (GTK_ENTRY (entry), mail_info.from);
   gimp_table_attach_aligned (GTK_TABLE (table), 0, row++,
-			     _("_From:"), 0.0, 0.5,
-			     entry, 1, FALSE);
+                             C_("email-address", "_From:"), 0.0, 0.5,
+                             entry, 1, FALSE);
   g_signal_connect (entry, "changed",
                     G_CALLBACK (mail_entry_callback),
                     mail_info.from);
@@ -479,8 +572,8 @@ save_dialog (void)
   gtk_entry_set_max_length (GTK_ENTRY (entry), BUFFER_SIZE - 1);
   gtk_entry_set_text (GTK_ENTRY (entry), mail_info.subject);
   gimp_table_attach_aligned (GTK_TABLE (table), 0, row++,
-			     _("S_ubject:"), 0.0, 0.5,
-			     entry, 1, FALSE);
+                             _("S_ubject:"), 0.0, 0.5,
+                             entry, 1, FALSE);
   g_signal_connect (entry, "changed",
                     G_CALLBACK (mail_entry_callback),
                     mail_info.subject);
@@ -499,7 +592,7 @@ save_dialog (void)
 
   g_signal_connect (text_buffer, "changed",
                     G_CALLBACK (mesg_body_callback),
-		    NULL);
+                    NULL);
 
   gtk_text_buffer_set_text (text_buffer, mail_info.comment, -1);
 
@@ -509,6 +602,7 @@ save_dialog (void)
   gtk_text_view_set_wrap_mode (GTK_TEXT_VIEW (text_view), GTK_WRAP_WORD);
   gtk_container_add (GTK_CONTAINER (scrolled_window), text_view);
   gtk_widget_show (text_view);
+#endif
 
   gtk_widget_show (dlg);
 
@@ -528,7 +622,68 @@ valid_file (const gchar *filename)
 }
 
 static gchar *
-find_content_type (const gchar *filename)
+find_extension (const gchar *filename)
+{
+  gchar *filename_copy;
+  gchar *ext;
+
+  /* we never free this copy - aren't we evil! */
+  filename_copy = g_strdup (filename);
+
+  /* find the extension, boy! */
+  ext = strrchr (filename_copy, '.');
+
+  while (TRUE)
+    {
+      if (!ext || ext[1] == '\0' || strchr (ext, G_DIR_SEPARATOR))
+        {
+          g_message (_("some sort of error with the file extension "
+                       "or lack thereof"));
+
+          return NULL;
+        }
+
+      if (0 != g_ascii_strcasecmp (ext, ".gz") &&
+          0 != g_ascii_strcasecmp (ext, ".bz2"))
+        {
+          return ext;
+        }
+      else
+        {
+          /* we found somehting, loop back, and look again */
+          *ext = 0;
+          ext = strrchr (filename_copy, '.');
+        }
+    }
+
+  g_free (filename_copy);
+
+  return ext;
+}
+
+static void
+mail_entry_callback (GtkWidget *widget,
+                     gchar     *data)
+{
+  g_strlcpy (data, gtk_entry_get_text (GTK_ENTRY (widget)), BUFFER_SIZE);
+}
+
+#ifdef SENDMAIL
+static void
+mesg_body_callback (GtkTextBuffer *buffer,
+                    gpointer       data)
+{
+  GtkTextIter start_iter;
+  GtkTextIter end_iter;
+
+  gtk_text_buffer_get_bounds (buffer, &start_iter, &end_iter);
+
+  g_free (mesg_body);
+  mesg_body = gtk_text_buffer_get_text (buffer, &start_iter, &end_iter, FALSE);
+}
+
+static gchar *
+sendmail_content_type (const gchar *filename)
 {
   /* This function returns a MIME Content-type: value based on the
      filename it is given.  */
@@ -562,9 +717,9 @@ find_content_type (const gchar *filename)
   while (type_mappings[i])
     {
       if (g_ascii_strcasecmp (ext, type_mappings[i]) == 0)
-	{
-	  return g_strdup (type_mappings[i + 1]);
-	}
+        {
+          return g_strdup (type_mappings[i + 1]);
+        }
 
       i += 2;
     }
@@ -572,70 +727,12 @@ find_content_type (const gchar *filename)
   return g_strdup_printf ("image/x-%s", ext);
 }
 
-static gchar *
-find_extension (const gchar *filename)
-{
-  gchar *filename_copy;
-  gchar *ext;
-
-  /* we never free this copy - aren't we evil! */
-  filename_copy = g_strdup (filename);
-
-  /* find the extension, boy! */
-  ext = strrchr (filename_copy, '.');
-
-  while (TRUE)
-    {
-      if (!ext || ext[1] == '\0' || strchr (ext, G_DIR_SEPARATOR))
-	{
-	  g_message (_("some sort of error with the file extension "
-                       "or lack thereof"));
-
-	  return NULL;
-	}
-
-      if (0 != g_ascii_strcasecmp (ext, ".gz") &&
-          0 != g_ascii_strcasecmp (ext, ".bz2"))
-	{
-	  return ext;
-	}
-      else
-	{
-	  /* we found somehting, loop back, and look again */
-	  *ext = 0;
-	  ext = strrchr (filename_copy, '.');
-	}
-    }
-
-  return ext;
-}
-
 static void
-mail_entry_callback (GtkWidget *widget,
-		     gchar     *data)
-{
-  g_strlcpy (data, gtk_entry_get_text (GTK_ENTRY (widget)), BUFFER_SIZE);
-}
-
-static void
-mesg_body_callback (GtkTextBuffer *buffer,
-		    gpointer       data)
-{
-  GtkTextIter start_iter;
-  GtkTextIter end_iter;
-
-  gtk_text_buffer_get_bounds (buffer, &start_iter, &end_iter);
-
-  g_free (mesg_body);
-  mesg_body = gtk_text_buffer_get_text (buffer, &start_iter, &end_iter, FALSE);
-}
-
-static void
-create_headers (FILE *mailpipe)
+sendmail_create_headers (FILE *mailpipe)
 {
   /* create all the mail header stuff. Feel free to add your own */
   /* It is advisable to leave the X-Mailer header though, as     */
-  /* there is a possibilty of a Gimp mail scanner/reader in the  */
+  /* there is a possibility of a Gimp mail scanner/reader in the  */
   /* future. It will probabaly need that header.                 */
 
   fprintf (mailpipe, "To: %s \n", mail_info.receipt);
@@ -643,7 +740,7 @@ create_headers (FILE *mailpipe)
   if (strlen (mail_info.from) > 0)
     fprintf (mailpipe, "From: %s \n", mail_info.from);
 
-  fprintf (mailpipe, "X-Mailer: GIMP Useless Mail Plug-In %s\n", GIMP_VERSION);
+  fprintf (mailpipe, "X-Mailer: GIMP Useless Mail plug-in %s\n", GIMP_VERSION);
 
   fprintf (mailpipe, "MIME-Version: 1.0\n");
   fprintf (mailpipe, "Content-type: multipart/mixed; "
@@ -660,7 +757,7 @@ create_headers (FILE *mailpipe)
   fprintf (mailpipe, "\n\n");
 
   {
-    gchar *content = find_content_type (mail_info.filename);
+    gchar *content = sendmail_content_type (mail_info.filename);
 
     fprintf (mailpipe, "--GUMP-MIME-boundary\n");
     fprintf (mailpipe, "Content-type: %s\n", content);
@@ -674,9 +771,9 @@ create_headers (FILE *mailpipe)
 }
 
 static gboolean
-to64 (const gchar  *filename,
-      FILE         *outfile,
-      GError      **error)
+sendmail_to64 (const gchar  *filename,
+               FILE         *outfile,
+               GError      **error)
 {
   GMappedFile  *infile;
   const guchar *in;
@@ -707,7 +804,7 @@ to64 (const gchar  *filename,
   bytes = g_base64_encode_close (TRUE, out, &state, &save);
   fwrite (out, 1, bytes, outfile);
 
-  g_mapped_file_free (infile);
+  g_mapped_file_unref (infile);
 
   return TRUE;
 }
@@ -719,8 +816,10 @@ sendmail_pipe (gchar **cmd,
   gint    fd;
   GError *err = NULL;
 
-  if (! g_spawn_async_with_pipes (NULL, cmd, NULL, G_SPAWN_DO_NOT_REAP_CHILD,
-	                          NULL, NULL, pid, &fd, NULL, NULL, &err))
+  if (! g_spawn_async_with_pipes (NULL, cmd, NULL,
+                                  G_SPAWN_DO_NOT_REAP_CHILD |
+                                  G_SPAWN_SEARCH_PATH,
+                                  NULL, NULL, pid, &fd, NULL, NULL, &err))
     {
       g_message (_("Could not start sendmail (%s)"), err->message);
       g_error_free (err);
@@ -731,3 +830,4 @@ sendmail_pipe (gchar **cmd,
 
   return fdopen (fd, "wb");
 }
+#endif
