@@ -5,9 +5,9 @@
  * You can contact me at quartic@polloux.fciencias.unam.mx
  * You can contact the original GIMP authors at gimp@xcf.berkeley.edu
  *
- * This program is free software; you can redistribute it and/or modify
+ * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -16,8 +16,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -48,11 +47,12 @@
 
 #include "libgimp/stdplugins-intl.h"
 
-#include "pagecurl-icons.h"
+#include "pagecurl-icons.c"
 
 
 #define PLUG_IN_PROC    "plug-in-pagecurl"
 #define PLUG_IN_BINARY  "pagecurl"
+#define PLUG_IN_ROLE    "gimp-pagecurl"
 #define PLUG_IN_VERSION "July 2004, 1.0"
 #define NGRADSAMPLES    256
 
@@ -124,7 +124,7 @@ static void       init_calculation     (gint32            drawable_id);
 static gint32     do_curl_effect       (gint32            drawable_id);
 static void       clear_curled_region  (gint32            drawable_id);
 static gint32     page_curl            (gint32            drawable_id);
-static guchar   * get_gradient_samples (gint32            drawable_id,
+static GimpRGB  * get_gradient_samples (gint32            drawable_id,
                                         gboolean          reverse);
 
 
@@ -143,23 +143,10 @@ static CurlParams curl;
 /* Image parameters */
 
 static gint32        image_id;
-static GimpDrawable *curl_layer;
-
-static const guint8 *curl_pixbufs[] =
-{
-  curl0,
-  curl1,
-  curl2,
-  curl3,
-  curl4,
-  curl5,
-  curl6,
-  curl7
-};
 
 static GtkWidget *curl_image = NULL;
 
-static gint   sel_x1, sel_y1, sel_x2, sel_y2;
+static gint   sel_x, sel_y;
 static gint   true_sel_width, true_sel_height;
 static gint   sel_width, sel_height;
 static gint   drawable_position;
@@ -183,8 +170,8 @@ static gdouble diagm_slope;
 
 /* User-configured parameters */
 
-static guchar fore_color[3];
-static guchar back_color[3];
+static GimpRGB fg_color;
+static GimpRGB bg_color;
 
 
 /***** Functions *****/
@@ -196,7 +183,7 @@ query (void)
 {
   static const GimpParamDef args[] =
   {
-    { GIMP_PDB_INT32,    "run-mode",    "Interactive (0), non-interactive (1)" },
+    { GIMP_PDB_INT32,    "run-mode",    "The run mode { RUN-INTERACTIVE (0), RUN-NONINTERACTIVE (1) }" },
     { GIMP_PDB_IMAGE,    "image",       "Input image"                          },
     { GIMP_PDB_DRAWABLE, "drawable",    "Input drawable"                       },
     { GIMP_PDB_INT32,    "colors",      "FG- and BG-Color (0), Current gradient (1), Current gradient reversed (2)" },
@@ -213,18 +200,18 @@ query (void)
   };
 
   gimp_install_procedure (PLUG_IN_PROC,
-			  N_("Curl up one of the image corners"),
-			  "This plug-in creates a pagecurl-effect.",
-			  "Federico Mena Quintero and Simon Budig",
-			  "Federico Mena Quintero and Simon Budig",
-			  PLUG_IN_VERSION,
-			  N_("_Pagecurl..."),
-			  "RGB*, GRAY*",
-			  GIMP_PLUGIN,
-			  G_N_ELEMENTS (args),
-			  G_N_ELEMENTS (return_vals),
-			  args,
-			  return_vals);
+                          N_("Curl up one of the image corners"),
+                          "This plug-in creates a pagecurl-effect.",
+                          "Federico Mena Quintero and Simon Budig",
+                          "Federico Mena Quintero and Simon Budig",
+                          PLUG_IN_VERSION,
+                          N_("_Pagecurl..."),
+                          "RGB*, GRAY*",
+                          GIMP_PLUGIN,
+                          G_N_ELEMENTS (args),
+                          G_N_ELEMENTS (return_vals),
+                          args,
+                          return_vals);
 
   gimp_plugin_menu_register (PLUG_IN_PROC, "<Image>/Filters/Distorts");
 }
@@ -244,6 +231,7 @@ run (const gchar      *name,
   run_mode = param[0].data.d_int32;
 
   INIT_I18N ();
+  gegl_init (NULL, NULL);
 
   set_default_params ();
 
@@ -262,51 +250,53 @@ run (const gchar      *name,
   drawable_id = param[2].data.d_drawable;
   image_id = param[1].data.d_image;
 
-  if (gimp_drawable_is_rgb (drawable_id)
-       || gimp_drawable_is_gray (drawable_id))
+  if ((gimp_drawable_is_rgb (drawable_id) ||
+       gimp_drawable_is_gray (drawable_id)) &&
+      gimp_drawable_mask_intersect (drawable_id, &sel_x, &sel_y,
+                                    &true_sel_width, &true_sel_height))
     {
       switch (run_mode)
-	{
-	case GIMP_RUN_INTERACTIVE:
-	  /*  First acquire information with a dialog  */
-	  if (! dialog ())
-	    return;
-	  break;
+        {
+        case GIMP_RUN_INTERACTIVE:
+          /*  First acquire information with a dialog  */
+          if (! dialog ())
+            return;
+          break;
 
-	case GIMP_RUN_NONINTERACTIVE:
-	  /*  Make sure all the arguments are there!  */
-	  if (nparams != 7)
-	    status = GIMP_PDB_CALLING_ERROR;
+        case GIMP_RUN_NONINTERACTIVE:
+          /*  Make sure all the arguments are there!  */
+          if (nparams != 7)
+            status = GIMP_PDB_CALLING_ERROR;
 
-	  if (status == GIMP_PDB_SUCCESS)
-	    {
+          if (status == GIMP_PDB_SUCCESS)
+            {
               curl.colors      = CLAMP (param[3].data.d_int32,
                                         0, CURL_COLORS_LAST);
               curl.edge        = CLAMP (param[4].data.d_int32,
                                         CURL_EDGE_FIRST, CURL_EDGE_LAST);
               curl.orientation = CLAMP (param[5].data.d_int32,
                                         0, CURL_ORIENTATION_LAST);
-	      curl.shade       = param[6].data.d_int32 ? TRUE : FALSE;
-	    }
-	  break;
+              curl.shade       = param[6].data.d_int32 ? TRUE : FALSE;
+            }
+          break;
 
-	case GIMP_RUN_WITH_LAST_VALS:
-	  break;
+        case GIMP_RUN_WITH_LAST_VALS:
+          break;
 
-	default:
-	  break;
-	}
+        default:
+          break;
+        }
 
       if (status == GIMP_PDB_SUCCESS)
-	{
-	  values[1].data.d_layer = page_curl (drawable_id);
+        {
+          values[1].data.d_layer = page_curl (drawable_id);
 
-	  if (run_mode != GIMP_RUN_NONINTERACTIVE)
+          if (run_mode != GIMP_RUN_NONINTERACTIVE)
             gimp_displays_flush ();
 
-	  if (run_mode == GIMP_RUN_INTERACTIVE)
+          if (run_mode == GIMP_RUN_INTERACTIVE)
             gimp_set_data (PLUG_IN_PROC, &curl, sizeof (CurlParams));
-	}
+        }
     }
   else
     /* Sorry - no indexed/noalpha images */
@@ -339,35 +329,35 @@ run (const gchar      *name,
 
 static inline gboolean
 left_of_diagl (gdouble x,
-	       gdouble y)
+               gdouble y)
 {
   return (x < (sel_width + (y - sel_height) * diagl_slope));
 }
 
 static inline gboolean
 right_of_diagr (gdouble x,
-		gdouble y)
+                gdouble y)
 {
   return (x > (sel_width + (y - sel_height) * diagr_slope));
 }
 
 static inline gboolean
 below_diagb (gdouble x,
-	     gdouble y)
+             gdouble y)
 {
   return (y < (right_tangent.y + (x - right_tangent.x) * diagb_slope));
 }
 
 static inline gboolean
 right_of_diagm (gdouble x,
-		gdouble y)
+                gdouble y)
 {
   return (x > (sel_width + (y - sel_height) * diagm_slope));
 }
 
 static inline gboolean
 inside_circle (gdouble x,
-	       gdouble y)
+               gdouble y)
 {
   x -= center.x;
   y -= center.y;
@@ -391,9 +381,9 @@ set_default_params (void)
 
 static void
 dialog_scale_update (GtkAdjustment *adjustment,
-		     gdouble       *value)
+                     gdouble       *value)
 {
-  *value = ((gdouble) adjustment->value) / 100.0;
+  *value = ((gdouble) gtk_adjustment_get_value (adjustment)) / 100.0;
 }
 
 static void
@@ -401,6 +391,7 @@ curl_pixbuf_update (void)
 {
   GdkPixbuf *pixbuf;
   gint       index;
+  gchar     *resource;
 
   switch (curl.edge)
     {
@@ -414,7 +405,11 @@ curl_pixbuf_update (void)
 
   index += curl.orientation * 4;
 
-  pixbuf = gdk_pixbuf_new_from_inline (-1, curl_pixbufs[index], FALSE, NULL);
+  resource = g_strdup_printf ("/org/gimp/pagecurl-icons/curl%c.png",
+                              '0' + index);
+  pixbuf = gdk_pixbuf_new_from_resource (resource, NULL);
+  g_free (resource);
+
   gtk_image_set_from_pixbuf (GTK_IMAGE (curl_image), pixbuf);
   g_object_unref (pixbuf);
 }
@@ -437,14 +432,14 @@ dialog (void)
 
   gimp_ui_init (PLUG_IN_BINARY, FALSE);
 
-  dialog = gimp_dialog_new (_("Pagecurl Effect"), PLUG_IN_BINARY,
+  dialog = gimp_dialog_new (_("Pagecurl Effect"), PLUG_IN_ROLE,
                             NULL, 0,
-			    gimp_standard_help_func, PLUG_IN_PROC,
+                            gimp_standard_help_func, PLUG_IN_PROC,
 
-			    GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-			    GTK_STOCK_OK,     GTK_RESPONSE_OK,
+                            _("_Cancel"), GTK_RESPONSE_CANCEL,
+                            _("_OK"),     GTK_RESPONSE_OK,
 
-			    NULL);
+                            NULL);
 
   gtk_dialog_set_alternative_button_order (GTK_DIALOG (dialog),
                                            GTK_RESPONSE_OK,
@@ -453,9 +448,9 @@ dialog (void)
 
   gimp_window_set_transient (GTK_WINDOW (dialog));
 
-  vbox = gtk_vbox_new (FALSE, 12);
+  vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 12);
   gtk_container_set_border_width (GTK_CONTAINER (vbox), 12);
-  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox),
+  gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (dialog))),
                       vbox, TRUE, TRUE, 0);
   gtk_widget_show (vbox);
 
@@ -524,7 +519,8 @@ dialog (void)
   frame = gimp_frame_new (_("Curl Orientation"));
   gtk_box_pack_start (GTK_BOX (vbox), frame, FALSE, FALSE, 0);
 
-  hbox = gtk_hbox_new (TRUE, 6);
+  hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 6);
+  gtk_box_set_homogeneous (GTK_BOX (hbox), TRUE);
   gtk_container_add (GTK_CONTAINER (frame), hbox);
 
   {
@@ -577,19 +573,19 @@ dialog (void)
   combo = g_object_new (GIMP_TYPE_INT_COMBO_BOX, NULL);
 
   gimp_int_combo_box_prepend (GIMP_INT_COMBO_BOX (combo),
-                              GIMP_INT_STORE_VALUE,    CURL_COLORS_GRADIENT_REVERSE,
-                              GIMP_INT_STORE_LABEL,    _("Current gradient (reversed)"),
-                              GIMP_INT_STORE_STOCK_ID, GIMP_STOCK_GRADIENT,
+                              GIMP_INT_STORE_VALUE,     CURL_COLORS_GRADIENT_REVERSE,
+                              GIMP_INT_STORE_LABEL,     _("Current gradient (reversed)"),
+                              GIMP_INT_STORE_ICON_NAME, GIMP_ICON_GRADIENT,
                               -1);
   gimp_int_combo_box_prepend (GIMP_INT_COMBO_BOX (combo),
-                              GIMP_INT_STORE_VALUE,    CURL_COLORS_GRADIENT,
-                              GIMP_INT_STORE_LABEL,    _("Current gradient"),
-                              GIMP_INT_STORE_STOCK_ID, GIMP_STOCK_GRADIENT,
+                              GIMP_INT_STORE_VALUE,     CURL_COLORS_GRADIENT,
+                              GIMP_INT_STORE_LABEL,     _("Current gradient"),
+                              GIMP_INT_STORE_ICON_NAME, GIMP_ICON_GRADIENT,
                               -1);
   gimp_int_combo_box_prepend (GIMP_INT_COMBO_BOX (combo),
-                              GIMP_INT_STORE_VALUE,    CURL_COLORS_FG_BG,
-                              GIMP_INT_STORE_LABEL,    _("Foreground / background colors"),
-                              GIMP_INT_STORE_STOCK_ID, GIMP_STOCK_DEFAULT_COLORS,
+                              GIMP_INT_STORE_VALUE,     CURL_COLORS_FG_BG,
+                              GIMP_INT_STORE_LABEL,     _("Foreground / background colors"),
+                              GIMP_INT_STORE_ICON_NAME, GIMP_ICON_COLORS_DEFAULT,
                               -1);
 
   gtk_box_pack_start (GTK_BOX (vbox), combo, FALSE, FALSE, 0);
@@ -633,7 +629,6 @@ init_calculation (gint32 drawable_id)
   GimpVector2  v1, v2;
   gint32      *image_layers;
   gint32       nlayers;
-  GimpRGB      color;
 
   gimp_layer_add_alpha (drawable_id);
 
@@ -644,16 +639,9 @@ init_calculation (gint32 drawable_id)
   image_layers = gimp_image_get_layers (image_id, &nlayers);
   drawable_position = 0;
   while (drawable_position < nlayers &&
-	 image_layers[drawable_position] != drawable_id)
+         image_layers[drawable_position] != drawable_id)
     drawable_position++;
 
-  /* Get the bounds of the active selection */
-  gimp_drawable_mask_bounds (drawable_id,
-			     &sel_x1, &sel_y1, &sel_x2, &sel_y2);
-
-  true_sel_width = sel_x2 - sel_x1;
-  true_sel_height = sel_y2 - sel_y1;
-
   switch (curl.orientation)
     {
     case CURL_ORIENTATION_VERTICAL:
@@ -685,11 +673,11 @@ init_calculation (gint32 drawable_id)
   gimp_vector2_sub (&v1, &left_tangent, &center);
   gimp_vector2_set (&v2, sel_width - center.x, sel_height - center.y);
   angle = -2.0 * acos (gimp_vector2_inner_product (&v1, &v2) /
-		       (gimp_vector2_length (&v1) *
+                       (gimp_vector2_length (&v1) *
                         gimp_vector2_length (&v2)));
   gimp_vector2_set (&right_tangent,
-		    v1.x * cos (angle) + v1.y * -sin (angle),
-		    v1.x * sin (angle) + v1.y * cos (angle));
+                    v1.x * cos (angle) + v1.y * -sin (angle),
+                    v1.x * sin (angle) + v1.y * cos (angle));
   gimp_vector2_add (&right_tangent, &right_tangent, &center);
 
   /* Slopes */
@@ -702,11 +690,8 @@ init_calculation (gint32 drawable_id)
 
   /* Colors */
 
-  gimp_context_get_foreground (&color);
-  gimp_rgb_get_uchar (&color, &fore_color[0], &fore_color[1], &fore_color[2]);
-
-  gimp_context_get_background (&color);
-  gimp_rgb_get_uchar (&color, &back_color[0], &back_color[1], &back_color[2]);
+  gimp_context_get_foreground (&fg_color);
+  gimp_context_get_background (&bg_color);
 }
 
 static gint32
@@ -715,59 +700,57 @@ do_curl_effect (gint32 drawable_id)
   gint          x = 0;
   gint          y = 0;
   gboolean      color_image;
-  gint          x1, y1, k;
-  guint         alpha_pos, progress, max_progress;
-  gdouble       intensity, alpha, beta;
+  gint          x1, y1;
+  guint         progress, max_progress;
+  gdouble       intensity, alpha;
   GimpVector2   v, dl, dr;
   gdouble       dl_mag, dr_mag, angle, factor;
-  guchar       *pp, *dest, fore_grayval, back_grayval;
-  guchar       *gradsamp;
-  GimpPixelRgn  dest_rgn;
-  gpointer      pr;
+  GeglBuffer   *curl_buffer;
   gint32        curl_layer_id;
-  guchar       *grad_samples  = NULL;
+  GimpRGB      *grad_samples = NULL;
+  gint          width, height, n_ch;
+  GeglRectangle *roi;
+  GeglBufferIterator *iter;
+  const Babl         *format;
 
   color_image = gimp_drawable_is_rgb (drawable_id);
 
-  curl_layer =
-    gimp_drawable_get (gimp_layer_new (image_id,
-				       _("Curl Layer"),
-				       true_sel_width,
-				       true_sel_height,
-				       color_image ?
-                                       GIMP_RGBA_IMAGE : GIMP_GRAYA_IMAGE,
-				       100, GIMP_NORMAL_MODE));
+  curl_layer_id = gimp_layer_new (image_id,
+                                  _("Curl Layer"),
+                                  true_sel_width,
+                                  true_sel_height,
+                                  color_image ?
+                                  GIMP_RGBA_IMAGE : GIMP_GRAYA_IMAGE,
+                                  100, GIMP_LAYER_MODE_NORMAL_LEGACY);
 
-  curl_layer_id = curl_layer->drawable_id;
-
-  gimp_image_add_layer (image_id, curl_layer->drawable_id, drawable_position);
-  gimp_drawable_fill (curl_layer->drawable_id, GIMP_TRANSPARENT_FILL);
+  gimp_image_insert_layer (image_id, curl_layer_id,
+                           gimp_item_get_parent (drawable_id),
+                           drawable_position);
+  gimp_drawable_fill (curl_layer_id, GIMP_FILL_TRANSPARENT);
 
   gimp_drawable_offsets (drawable_id, &x1, &y1);
-  gimp_layer_set_offsets (curl_layer->drawable_id, sel_x1 + x1, sel_y1 + y1);
-  gimp_tile_cache_ntiles (2 * (curl_layer->width / gimp_tile_width () + 1));
+  gimp_layer_set_offsets (curl_layer_id, sel_x + x1, sel_y + y1);
 
-  gimp_pixel_rgn_init (&dest_rgn, curl_layer,
-		       0, 0, true_sel_width, true_sel_height, TRUE, TRUE);
+  curl_buffer = gimp_drawable_get_shadow_buffer (curl_layer_id);
+
+  width  = gegl_buffer_get_width (curl_buffer);
+  height = gegl_buffer_get_height (curl_buffer);
+  format = babl_format ("R'G'B'A float");
+  n_ch   = babl_format_get_n_components (format);
+
+  iter = gegl_buffer_iterator_new (curl_buffer,
+                                   GEGL_RECTANGLE (0, 0, width, height), 0,
+                                   format,
+                                   GEGL_ACCESS_WRITE, GEGL_ABYSS_NONE);
 
   /* Init shade_under */
   gimp_vector2_set (&dl, -sel_width, -sel_height);
   dl_mag = gimp_vector2_length (&dl);
   gimp_vector2_set (&dr,
-		    -(sel_width - right_tangent.x),
-		    -(sel_height - right_tangent.y));
+                    -(sel_width - right_tangent.x),
+                    -(sel_height - right_tangent.y));
   dr_mag = gimp_vector2_length (&dr);
   alpha = acos (gimp_vector2_inner_product (&dl, &dr) / (dl_mag * dr_mag));
-  beta = alpha / 2;
-
-  /* Init shade_curl */
-
-  fore_grayval = GIMP_RGB_LUMINANCE (fore_color[0],
-                                     fore_color[1],
-                                     fore_color[2]) + 0.5;
-  back_grayval = GIMP_RGB_LUMINANCE (back_color[0],
-                                     back_color[1],
-                                     back_color[2]) + 0.5;
 
   /* Gradient Samples */
   switch (curl.colors)
@@ -775,142 +758,117 @@ do_curl_effect (gint32 drawable_id)
     case CURL_COLORS_FG_BG:
       break;
     case CURL_COLORS_GRADIENT:
-      grad_samples = get_gradient_samples (curl_layer->drawable_id, FALSE);
+      grad_samples = get_gradient_samples (curl_layer_id, FALSE);
       break;
     case CURL_COLORS_GRADIENT_REVERSE:
-      grad_samples = get_gradient_samples (curl_layer->drawable_id, TRUE);
+      grad_samples = get_gradient_samples (curl_layer_id, TRUE);
       break;
     }
 
   max_progress = 2 * sel_width * sel_height;
   progress = 0;
 
-  alpha_pos = dest_rgn.bpp - 1;
-
   /* Main loop */
-  for (pr = gimp_pixel_rgns_register (1, &dest_rgn);
-       pr != NULL;
-       pr = gimp_pixel_rgns_process (pr))
+  while (gegl_buffer_iterator_next (iter))
     {
-      dest = dest_rgn.data;
+      gfloat *dest;
 
-      for (y1 = dest_rgn.y; y1 < dest_rgn.y + dest_rgn.h; y1++)
-	{
-	  pp = dest;
-	  for (x1 = dest_rgn.x; x1 < dest_rgn.x + dest_rgn.w; x1++)
-	    {
-	      /* Map coordinates to get the curl correct... */
+      roi = &iter->roi[0];
+      dest = (gfloat *) iter->data[0];
+
+      for (y1 = roi->y; y1 < roi->y + roi->height; y1++)
+        {
+          GimpRGB color;
+
+          for (x1 = roi->x; x1 < roi->x + roi->width; x1++)
+            {
+              /* Map coordinates to get the curl correct... */
               switch (curl.orientation)
                 {
                 case CURL_ORIENTATION_VERTICAL:
-		  x = CURL_EDGE_RIGHT (curl.edge) ? x1 : sel_width  - 1 - x1;
-		  y = CURL_EDGE_UPPER (curl.edge) ? y1 : sel_height - 1 - y1;
+                  x = CURL_EDGE_RIGHT (curl.edge) ? x1 : sel_width  - 1 - x1;
+                  y = CURL_EDGE_UPPER (curl.edge) ? y1 : sel_height - 1 - y1;
                   break;
 
                 case CURL_ORIENTATION_HORIZONTAL:
                   x = CURL_EDGE_LOWER (curl.edge) ? y1 : sel_width  - 1 - y1;
-		  y = CURL_EDGE_LEFT  (curl.edge) ? x1 : sel_height - 1 - x1;
+                  y = CURL_EDGE_LEFT  (curl.edge) ? x1 : sel_height - 1 - x1;
                   break;
-		}
+                }
 
-	      if (left_of_diagl (x, y))
-		{ /* uncurled region */
-		  for (k = 0; k <= alpha_pos; k++)
-		    pp[k] = 0;
-		}
-	      else if (right_of_diagr (x, y) ||
-		       (right_of_diagm (x, y) &&
-			below_diagb (x, y) &&
-			!inside_circle (x, y)))
-		{
-		  /* curled region */
-		  for (k = 0; k <= alpha_pos; k++)
-		    pp[k] = 0;
-		}
-	      else
-		{
-		  v.x = -(sel_width - x);
-		  v.y = -(sel_height - y);
-		  angle = acos (gimp_vector2_inner_product (&v, &dl) /
-				(gimp_vector2_length (&v) * dl_mag));
+              if (left_of_diagl (x, y))
+                { /* uncurled region: transparent black */
+                  gimp_rgba_set (&color, 0, 0, 0, 0);
+                }
+              else if (right_of_diagr (x, y) ||
+                       (right_of_diagm (x, y) &&
+                        below_diagb (x, y) &&
+                        !inside_circle (x, y)))
+                {
+                  /* curled region: transparent black */
+                  gimp_rgba_set (&color, 0, 0, 0, 0);
+                }
+              else
+                {
+                  v.x = -(sel_width - x);
+                  v.y = -(sel_height - y);
+                  angle = acos (gimp_vector2_inner_product (&v, &dl) /
+                                (gimp_vector2_length (&v) * dl_mag));
 
-		  if (inside_circle (x, y) || below_diagb (x, y))
-		    {
-		      /* Below the curl. */
-		      factor = angle / alpha;
-		      for (k = 0; k < alpha_pos; k++)
-			pp[k] = 0;
+                  if (inside_circle (x, y) || below_diagb (x, y))
+                    {
+                      /* Below the curl. */
+                      factor = angle / alpha;
+                      gimp_rgba_set (&color, 0, 0, 0, curl.shade ? factor : 0);
+                    }
+                  else
+                    {
+                      GimpRGB *gradrgb;
 
-		      pp[alpha_pos] = (curl.shade ?
-                                       (guchar) ((float) 255 * (float) factor) :
-                                       0);
-		    }
-		  else
-		    {
-		      /* On the curl */
+                      /* On the curl */
                       switch (curl.colors)
                         {
                         case CURL_COLORS_FG_BG:
-			  intensity = pow (sin (G_PI * angle / alpha), 1.5);
-			  if (color_image)
-			    {
-			      pp[0] = (intensity * back_color[0] +
-                                       (1.0 - intensity) * fore_color[0]);
-			      pp[1] = (intensity * back_color[1] +
-                                       (1.0 - intensity) * fore_color[1]);
-			      pp[2] = (intensity * back_color[2] +
-                                       (1.0 - intensity) * fore_color[2]);
-			    }
-			  else
-			    pp[0] = (intensity * back_grayval +
-                                     (1 - intensity) * fore_grayval);
-
-			  pp[alpha_pos] = (guchar) ((double) 255.99 *
-                                                    (1.0 - intensity *
-                                                     (1.0 - curl.opacity)));
+                          intensity = pow (sin (G_PI * angle / alpha), 1.5);
+                          gimp_rgba_set (&color,
+                                         intensity * bg_color.r + (1.0 - intensity) * fg_color.r,
+                                         intensity * bg_color.g + (1.0 - intensity) * fg_color.g,
+                                         intensity * bg_color.b + (1.0 - intensity) * fg_color.b,
+                                         (1.0 - intensity * (1.0 - curl.opacity)));
                           break;
 
                         case CURL_COLORS_GRADIENT:
                         case CURL_COLORS_GRADIENT_REVERSE:
-			  /* Calculate position in Gradient */
-                          intensity =
-                            (angle/alpha) + sin (G_PI*2 * angle/alpha) * 0.075;
+                          /* Calculate position in Gradient */
+                          intensity = (angle/alpha) + sin (G_PI*2 * angle/alpha) * 0.075;
 
-			  /* Check boundaries */
-			  intensity = CLAMP (intensity, 0.0, 1.0);
-			  gradsamp  = (grad_samples +
-                                       ((guint) (intensity * NGRADSAMPLES)) *
-                                       dest_rgn.bpp);
+                          gradrgb = grad_samples + ((guint) (intensity * NGRADSAMPLES));
 
-			  if (color_image)
-			    {
-			      pp[0] = gradsamp[0];
-			      pp[1] = gradsamp[1];
-			      pp[2] = gradsamp[2];
-			    }
-			  else
-			    pp[0] = gradsamp[0];
-
-			  pp[alpha_pos] =
-                            (guchar) ((double) gradsamp[alpha_pos] *
-                                      (1.0 - intensity * (1.0 - curl.opacity)));
+                          /* Check boundaries */
+                          intensity = CLAMP (intensity, 0.0, 1.0);
+                          color = *gradrgb;
+                          color.a = gradrgb->a * (1.0 - intensity * (1.0 - curl.opacity));
                           break;
                         }
-		    }
-		}
-	      pp += dest_rgn.bpp;
-	    }
-	  dest += dest_rgn.rowstride;
-	}
-      progress += dest_rgn.w * dest_rgn.h;
+                    }
+                }
+
+              dest[0] = color.r;
+              dest[1] = color.g;
+              dest[2] = color.b;
+              dest[3] = color.a;
+
+              dest += n_ch;
+            }
+        }
+      progress += roi->width * roi->height;
       gimp_progress_update ((double) progress / (double) max_progress);
     }
 
-  gimp_drawable_flush (curl_layer);
-  gimp_drawable_merge_shadow (curl_layer->drawable_id, FALSE);
-  gimp_drawable_update (curl_layer->drawable_id,
-			0, 0, curl_layer->width, curl_layer->height);
-  gimp_drawable_detach (curl_layer);
+  gimp_progress_update (0.5);
+  gegl_buffer_flush (curl_buffer);
+  gimp_drawable_merge_shadow (curl_layer_id, FALSE);
+  gimp_drawable_update (curl_layer_id, 0, 0, width, height);
 
   g_free (grad_samples);
 
@@ -922,94 +880,96 @@ do_curl_effect (gint32 drawable_id)
 static void
 clear_curled_region (gint32 drawable_id)
 {
-  GimpPixelRgn  src_rgn, dest_rgn;
-  gpointer      pr;
   gint          x = 0;
   gint          y = 0;
-  guint         x1, y1, i;
-  guchar       *dest, *src, *pp, *sp;
-  guint         alpha_pos, progress, max_progress;
-  GimpDrawable *drawable;
+  guint         x1, y1;
+  gfloat       *dest, *src;
+  guint         progress, max_progress;
+  GeglBuffer   *buf;
+  GeglBuffer   *shadow_buf;
+  GeglRectangle *roi;
+  GeglBufferIterator *iter;
+  const Babl   *format;
+  gint          width, height, bpp, n_ch;
+  gint          buf_index;
 
   max_progress = 2 * sel_width * sel_height;
   progress = max_progress / 2;
 
-  drawable = gimp_drawable_get (drawable_id);
+  buf        = gimp_drawable_get_buffer (drawable_id);
+  shadow_buf = gimp_drawable_get_shadow_buffer (drawable_id);
+  width  = gegl_buffer_get_width (buf);
+  height = gegl_buffer_get_height (buf);
 
-  gimp_tile_cache_ntiles (2 * (drawable->width / gimp_tile_width () + 1));
-  gimp_pixel_rgn_init (&src_rgn, drawable,
-		       sel_x1, sel_y1, true_sel_width, true_sel_height,
-		       FALSE, FALSE);
-  gimp_pixel_rgn_init (&dest_rgn, drawable,
-		       sel_x1, sel_y1, true_sel_width, true_sel_height,
-		       TRUE, TRUE);
-  alpha_pos = dest_rgn.bpp - 1;
+  format = babl_format ("R'G'B'A float");
+  bpp    = babl_format_get_bytes_per_pixel (format);
+  n_ch   = babl_format_get_n_components (format);
 
-  for (pr = gimp_pixel_rgns_register (2, &dest_rgn, &src_rgn);
-       pr != NULL;
-       pr = gimp_pixel_rgns_process (pr))
+  iter = gegl_buffer_iterator_new (shadow_buf,
+                                   GEGL_RECTANGLE (0, 0, width, height), 0,
+                                   format,
+                                   GEGL_ACCESS_WRITE, GEGL_ABYSS_NONE);
+  buf_index = gegl_buffer_iterator_add (iter, buf, NULL, 0,
+                                        format,
+                                        GEGL_ACCESS_READ, GEGL_ABYSS_NONE);
+
+  while (gegl_buffer_iterator_next (iter))
     {
-      dest = dest_rgn.data;
-      src = src_rgn.data;
+      roi  = &iter->roi[0];
+      dest = iter->data[0];
+      src  = iter->data[buf_index];
 
-      for (y1 = dest_rgn.y; y1 < dest_rgn.y + dest_rgn.h; y1++)
-	{
-	  sp = src;
-	  pp = dest;
+      memcpy (dest, src, roi->width * roi->height * bpp);
 
-	  for (x1 = dest_rgn.x; x1 < dest_rgn.x + dest_rgn.w; x1++)
-	    {
-	      /* Map coordinates to get the curl correct... */
+      for (y1 = roi->y; y1 < roi->y + roi->height; y1++)
+        {
+          for (x1 = roi->x; x1 < roi->x + roi->width; x1++)
+            {
+              /* Map coordinates to get the curl correct... */
               switch (curl.orientation)
                 {
                 case CURL_ORIENTATION_VERTICAL:
-		  x = (CURL_EDGE_RIGHT (curl.edge) ?
-                       x1 - sel_x1 : sel_width  - 1 - (x1 - sel_x1));
-		  y = (CURL_EDGE_UPPER (curl.edge) ?
-                       y1 - sel_y1 : sel_height - 1 - (y1 - sel_y1));
+                  x = (CURL_EDGE_RIGHT (curl.edge) ?
+                       x1 - sel_x : sel_width  - 1 - (x1 - sel_x));
+                  y = (CURL_EDGE_UPPER (curl.edge) ?
+                       y1 - sel_y : sel_height - 1 - (y1 - sel_y));
                   break;
 
                 case CURL_ORIENTATION_HORIZONTAL:
-		  x = (CURL_EDGE_LOWER (curl.edge) ?
-                       y1 - sel_y1 : sel_width - 1 - (y1 - sel_y1));
+                  x = (CURL_EDGE_LOWER (curl.edge) ?
+                       y1 - sel_y : sel_width - 1 - (y1 - sel_y));
                   y = (CURL_EDGE_LEFT (curl.edge)  ?
-                       x1 - sel_x1 : sel_height - 1 - (x1 - sel_x1));
+                       x1 - sel_x : sel_height - 1 - (x1 - sel_x));
                   break;
                 }
 
-	      for (i = 0; i < alpha_pos; i++)
-		pp[i] = sp[i];
+              if (right_of_diagr (x, y) ||
+                  (right_of_diagm (x, y) &&
+                   below_diagb (x, y) &&
+                   !inside_circle (x, y)))
+                {
+                  /* Right of the curl: Alpha = 0 */
+                  dest[0] = src[0];
+                  dest[1] = src[1];
+                  dest[2] = src[2];
+                  dest[3] = 0.0;
+                }
 
-	      if (right_of_diagr (x, y) ||
-		  (right_of_diagm (x, y) &&
-		   below_diagb (x, y) &&
-		   !inside_circle (x, y)))
-		{
-		  /* Right of the curl */
-		  pp[alpha_pos] = 0;
-		}
-	      else
-		{
-		  pp[alpha_pos] = sp[alpha_pos];
-		}
+              dest += n_ch;
+              src += n_ch;
+            }
+        }
 
-	      pp += dest_rgn.bpp;
-	      sp += src_rgn.bpp;
-	    }
-
-	  src += src_rgn.rowstride;
-	  dest += dest_rgn.rowstride;
-	}
-
-      progress += dest_rgn.w * dest_rgn.h;
+      progress += roi->width * roi->height;
       gimp_progress_update ((double) progress / (double) max_progress);
     }
 
-  gimp_drawable_flush (drawable);
+  gimp_progress_update (1.0);
+  gegl_buffer_flush (shadow_buf);
   gimp_drawable_merge_shadow (drawable_id, TRUE);
   gimp_drawable_update (drawable_id,
-			sel_x1, sel_y1, true_sel_width, true_sel_height);
-  gimp_drawable_detach (drawable);
+                        sel_x, sel_y,
+                        true_sel_width, true_sel_height);
 }
 
 static gint32
@@ -1037,47 +997,32 @@ page_curl (gint32 drawable_id)
   Each sample has (gimp_drawable_bpp (drawable_id)) bytes.
   "ripped" from gradmap.c.
  */
-static guchar *
+static GimpRGB *
 get_gradient_samples (gint32    drawable_id,
                       gboolean  reverse)
 {
   gchar   *gradient_name;
-  gint     n_f_samples;
-  gdouble *f_samples, *f_samp;    /* float samples */
-  guchar  *b_samples, *b_samp;    /* byte samples */
-  gint     bpp, color, has_alpha, alpha;
-  gint     i, j;
+  gint     n_d_samples;
+  gdouble *d_samples = NULL;
+  GimpRGB *rgba;
+  gint     i;
 
   gradient_name = gimp_context_get_gradient ();
 
   gimp_gradient_get_uniform_samples (gradient_name, NGRADSAMPLES, reverse,
-                                     &n_f_samples, &f_samples);
+                                     &n_d_samples, &d_samples);
+
+  rgba = g_new0 (GimpRGB, NGRADSAMPLES);
+  for (i = 0; i < NGRADSAMPLES; i++)
+    {
+      gimp_rgba_set (rgba + i,
+                     d_samples[i*4 + 0],
+                     d_samples[i*4 + 1],
+                     d_samples[i*4 + 2],
+                     d_samples[i*4 + 3]);
+    }
 
   g_free (gradient_name);
 
-  bpp       = gimp_drawable_bpp (drawable_id);
-  color     = gimp_drawable_is_rgb (drawable_id);
-  has_alpha = gimp_drawable_has_alpha (drawable_id);
-  alpha     = (has_alpha ? bpp - 1 : bpp);
-
-  b_samples = g_new (guchar, NGRADSAMPLES * bpp);
-
-  for (i = 0; i < NGRADSAMPLES; i++)
-    {
-      b_samp = &b_samples[i * bpp];
-      f_samp = &f_samples[i * 4];
-
-      if (color)
-        for (j = 0; j < 3; j++)
-          b_samp[j] = f_samp[j] * 255;
-      else
-        b_samp[0] = GIMP_RGB_LUMINANCE (f_samp[0], f_samp[1], f_samp[2]) * 255;
-
-      if (has_alpha)
-        b_samp[alpha] = f_samp[3] * 255;
-    }
-
-  g_free (f_samples);
-
-  return b_samples;
+  return rgba;
 }
